function [data,flag,model,numpar,plotpar] = ifs_auxiliary(admin,data,flag,model,numpar,plotpar)
%
% Computes auxiliary variables and flags.
%

% Retrieve:
alpha = model.alpha;
c = model.c; % reservoir width to the left of the fault, m
d = model.d; % reservoir width to the right of the fault, m
G = model.G; % shear modulus, Pa
h = model.h; % reservoir heigth, m
nu = model.nu; % Poisson's ratio, -
phi = model.phi; % porosity, -
rho_f = model.rho_f; % fluid density, kg/m^3
rho_s = model.rho_s; % solid density, kg/m^3
t_f = model.t_f; % fault throw, m
theta = model.theta; % dip angle, rad
p_inc_ref = plotpar.p_inc_ref; % reference pressure used in image plots etc., Pa
dpdt_PU = numpar.dpdt_PU; % pressure drop per unit time (in practical units), MPa/yr 
N_cheb = numpar.N_cheb; % total number of integration points or Chebyshev polynomials 

% Compute new variables:
model.A = G/(2*pi*(1-nu)); % auxilary variable, Pa 
model.a = (h-t_f)/2; % height of reservoir bottom part (for positive x values) or reservoir ...
    % top part (for negative x values), m; see Fig. 1 in Jansen et al. (2019)
model.b = (h+t_f)/2; % height of reservoir top part (for positive x values) or reservoir ...
    % bottom part (for negative x values), m
rho_av = (1-phi)*rho_s + phi*rho_f; % average density, kg/m^3
c_s = sqrt(G/rho_av); % shear wave velocity, m/s    
model.eta_rad = G/(2*c_s); % radiation damping factor, Pa*s/m
numpar.dpdt = dpdt_PU*1e6/(365*24*3600); % pressure drop per unit time, Pa/s 
plotpar.C_ref = (1-2*nu)*alpha*p_inc_ref/(2*pi*(1-nu)); % C value for reference pressure ...
    % used in image plots etc.
if admin.input ~= 03
    model.K_tot = 0; %  K_tot is not needed except for Buijze et al. (2019) example
end
% if flag.int == 3 && flag.N_cheb == 0
    % Compute and store Chebyshev polynomials and Chebyshev points:
    N = N_cheb/2;
    [numpar.T_1,numpar.z_T_1] = ifs_cheb_matrix_T(N,N+1,1); % Note 1 column more!
    [numpar.T_2,numpar.z_T_2] = ifs_cheb_matrix_T(N,N+1,2); % Note 1 column more!
    [numpar.U_1,numpar.z_U_1] = ifs_cheb_matrix_U(N,N,1);
    [numpar.U_2,numpar.z_U_2] = ifs_cheb_matrix_U(N,N,2);
% end

% Set new flags:
if c == inf && d == inf % see also ifs_check.m
    flag.inf = 1; % infinite boundaries
else
    flag.inf = 0; % finite boundaries
end

if theta == pi/2
    flag.vert = 1; % vertical fault
else
    flag.vert = 0; % inclined fault
end

if flag.coupled == 0 
    flag.sim_type = 1; % uncoupled
else
    flag.sim_type = 2; % coupled
end
% Note: sim_type = 3 means that merging of patches has coccurred; see ifs_patch_bounds. 

if flag.depdep == 1 || flag.int == 3 % symmetric formulation not ...
    % available for depth-dependent initial pressure and stresses; forward simulation ...
    % with discrete Chebyshev polynomials 
    flag.sym = 0; 
else
    flag.sym = 1; % make use of symmetry around y = 0.
end

% Compute auxiliary plot data:
if flag.plot_diagn == 1 && flag.fric == 1 && flag.inf == 1 && admin.input ~= 03 
    y_reg = linspace(model.a-1,model.a+1,2000); % y values for plotting d_sigma_C_dy
    save = numpar.eta_reg;
    numpar.eta_reg = 0; % temporarily reset
    sigma_unreg = ifs_sigma(flag,model,numpar,0,0,p_inc_ref,0,y_reg,y_reg);
    sigma_C_unreg = sigma_unreg.C; 
    p_shift_unreg = sigma_unreg.p_shift;
    if admin.input ~= 03 % excludes Buijze et al. (2019) example 
        d_sigma_C_dy_unreg = ifs_d_sigma_C_dy(flag,model,numpar,p_inc_ref,y_reg);
    end
    y_pre = ifs_y_pre(flag,model,numpar,p_inc_ref);
    y_pre_unreg = y_pre(4);
    numpar.eta_reg = save; % restores original value
    sigma_reg = ifs_sigma(flag,model,numpar,0,0,p_inc_ref,0,y_reg,y_reg);
    sigma_C_reg = sigma_reg.C;
    p_shift_reg = sigma_reg.p_shift; 
    d_sigma_C_dy_reg = ifs_d_sigma_C_dy(flag,model,numpar,p_inc_ref,y_reg);
    y_0_deriv_reg = fzero(@(y_deriv) ifs_d_sigma_C_dy(flag,model,numpar,p_inc_ref,y_deriv),model.a);
    data.y_reg = y_reg; % y values of segment for plotting, m
    data.sigma_C_reg = sigma_C_reg; % regularized Coulomb stress value, Pa  
    data.sigma_C_unreg = sigma_C_unreg; % unregularized Coulomb stress value, Pa
    data.p_shift_reg = p_shift_reg; % p_shift to compute regularized effective stress, Pa  
    data.p_shift_unreg = p_shift_unreg; % p_shift to compute unregularized effective stress, Pa  
    data.y_pre_unreg = y_pre_unreg; % unregularized Coulomb stress zero, m 
    data.d_sigma_C_dy_reg = d_sigma_C_dy_reg; % regularized Coulomb stress derivative, Pa/m
    data.d_sigma_C_dy_unreg = d_sigma_C_dy_unreg; % unregularized Coulomb stress deriv. Pa/m
    data.y_0_deriv_reg = y_0_deriv_reg; % regularized Coulomb stress derivative zero, m
end
