function [T,z_j,z_i] = ifs_cheb_matrix_T(J,N,points_type,z_inp)
%
% Computes a J x N matrix with N first-kind Chebyshev polynomials as columns, i.e. T_0 to
% T_N-1, at J first-kind Chebyshev points z_j as rows. Also computes a set of complementary
% points z_i, mid-in between points z_j for use in numerical integration of singular integrals.
% Uses a recursive formulation.  
%
% The input variable "points_type" determines the location of the points z_j:
%   1 = zeros of T_(J+1)(z) where -1 < z < 1 (regular first-kind Chebyshev points)
%   2 = zeros of U_J(z) where -1 < z < 1 (regular second-kind Chebyshev points)
%   3 = not defined for first-kind Chebyshev polynomials 
%   4 = not defined for first-kind Chebyshev polynomials 
%   5 = zeros of sqrt(1-z^2) U_(J-2)(z) where -1 <= z <= 1 (modified second-kind Cheb. points)
%   6 = equidistantly distributed excluding the end points: -1 < z < 1 
%   7 = equidistantly distributed including the end points: -1 <= z <= 1 
%   8 = specified as z_inp through header
%
% Note: all Chebyshev points z_j in this implementation run from low to high, which is opposite
% to the formal definition. 
%
% Reference: Mason & Handscomb (2003). 
%

switch points_type
    case 1
        z_j = flip(cos( pi*((1:J)-0.5)/J ))';
        z_i = flip(cos( pi*((1:J-1))/J ))';
    case 2
        z_j = flip(cos( pi*(1:J)/(J+1) ))';
    case 3
        error('Points type 3 not defined for ifs_cheb_matrix_T.')
    case 4
        error('Points type 4 not defined for ifs_cheb_matrix_T.')
    case 5
        z_j = flip(cos( pi*(0:J-1)/(J-1) ))';
    case 6
        help = linspace(-1,1,2*J+1)';
        z_j = help(2:2:end-1); % half spacing at beginning and end
    case 7
        z_j = linspace(-1,1,J)';
    case 8
%        z_j = isrow(z_inp)*z_inp' + iscolumn(z_inp)*z_inp;
        z_j = z_inp;
        if length(z_j) ~= J
            error('Discrepancy between length(z_j) and J in ifs_cheb_matrix_T.') 
        end
end
T = zeros(J,N);
T(:,1) = 1; % T_0 % Chebyshev polynomials start counting at 0
T(:,2) = z_j; % T_1
for i = 3:N
    T(:,i) = 2*z_j.*T(:,i-1)-T(:,i-2); % T_2,...,T_N-1
end
