function [C0,C0_short,C1,C1_short,D0,D1,D2,D3] = ifs_compute_CD(flag,model,numpar,delta,...
    p_inc,y_delta,y_loc)
%
% Computes values of coefficients C0, C1 and D0 up to D3 for coupled simulation. 
% Version with second-kind Chebyshev polynomials to compute coefficients bb in linear system.
%
a = model.a; % m
N_bot = numpar.N_bot; % -
N_top = numpar.N_top; 
switch flag.int
    case {1,2} % inverse method: Gauss-Chebyshev quadrature or Matlab's variable-support ...
        % trapezium rule
        % 1) first-kind Chebyshev points: 
        [~,w_j_bot_T,y_i_bot_T,y_j_bot_T] = ifs_GaussCheb(00,N_bot,y_loc(1),y_loc(2));
        [~,w_j_top_T,y_i_top_T,y_j_top_T] = ifs_GaussCheb(00,N_top,y_loc(3),y_loc(4));
        % 2) second-kind Chebyshev points: 
        [~,w_j_bot_U,y_i_bot_U,y_j_bot_U] = ifs_GaussCheb(11,N_bot,y_loc(1),y_loc(2));
        [~,w_j_top_U,y_i_top_U,y_j_top_U] = ifs_GaussCheb(11,N_top,y_loc(3),y_loc(4));
end
% 1) first-kind Chebyshev points: 
sigma_bot_T = ifs_sigma(flag,model,numpar,delta,0,p_inc,y_delta,y_j_bot_T,y_j_bot_T); % Pa
sigma_top_T = ifs_sigma(flag,model,numpar,delta,0,p_inc,y_delta,y_j_top_T,y_j_top_T); 
Psi_2_bot_T = -ifs_Psi(y_j_bot_T,y_loc(1:4));
Psi_2_top_T =  ifs_Psi(y_j_top_T,y_loc(1:4));
% 2) second-kind Chebyshev points: 
sigma_bot_U = ifs_sigma(flag,model,numpar,delta,0,p_inc,y_delta,y_j_bot_U,y_j_bot_U); % Pa
sigma_top_U = ifs_sigma(flag,model,numpar,delta,0,p_inc,y_delta,y_j_top_U,y_j_top_U); 
% Psi_1_bot_T = -ifs_Psi(y_i_bot_T,y_loc(1:4)); % m^2
% Psi_1_top_T =  ifs_Psi(y_i_top_T,y_loc(1:4));
Psi_1_bot_U = -ifs_Psi(y_i_bot_U,y_loc(1:4)); % m^2
Psi_1_top_U =  ifs_Psi(y_i_top_U,y_loc(1:4));
Psi_2_bot_U = -ifs_Psi(y_j_bot_U,y_loc(1:4));
Psi_2_top_U =  ifs_Psi(y_j_top_U,y_loc(1:4));

% Compute C0 and C1 (short cut method):
switch flag.int
    case 1 % inverse method: Gauss-Chebyshev quadrature
        F_bot_T = -sigma_bot_T.C ./ ...
            -sqrt( abs( (y_j_bot_T-y_loc(3)) .* (y_j_bot_T-y_loc(4)) ) ); % Pa/m^2
        F_top_T = -sigma_top_T.C ./ ...
             sqrt( abs( (y_j_top_T-y_loc(1)) .* (y_j_top_T-y_loc(2)) ) );        
        I0_bot = sum(                F_bot_T .* w_j_bot_T); % Pa/m
        I1_bot = sum(y_j_bot_T    .* F_bot_T .* w_j_bot_T); % Pa
        I2_bot = sum(y_j_bot_T.^2 .* F_bot_T .* w_j_bot_T); % Pa*m
        I3_bot = sum(y_j_bot_T.^3 .* F_bot_T .* w_j_bot_T); % Pa*m^2
        I0_top = sum(                F_top_T .* w_j_top_T); 
        I1_top = sum(y_j_top_T    .* F_top_T .* w_j_top_T); 
        I2_top = sum(y_j_top_T.^2 .* F_top_T .* w_j_top_T); 
        I3_top = sum(y_j_top_T.^3 .* F_top_T .* w_j_top_T); 
    case 2 % inverse method: Matlab's variable-support trapezium rule
        % 1) first-kind Chebyshev points: 
        F_bot_T = -sigma_bot_T.C ./ Psi_2_bot_T; % Pa/m^2
        F_top_T = -sigma_top_T.C ./ Psi_2_top_T; 
        % 2) second-kind Chebyshev points: 
        I0_bot = trapz(y_j_bot_T,                 F_bot_T); % Pa/m
        I1_bot = trapz(y_j_bot_T, y_j_bot_T    .* F_bot_T); % Pa
        I2_bot = trapz(y_j_bot_T, y_j_bot_T.^2 .* F_bot_T); % Pa*m
        I3_bot = trapz(y_j_bot_T, y_j_bot_T.^3 .* F_bot_T); % Pa*m^2
        I0_top = trapz(y_j_top_T,                 F_top_T); 
        I1_top = trapz(y_j_top_T, y_j_top_T    .* F_top_T);
        I2_top = trapz(y_j_top_T, y_j_top_T.^2 .* F_top_T); 
        I3_top = trapz(y_j_top_T, y_j_top_T.^3 .* F_top_T); 
end
I0 = I0_bot + I0_top; % should be equal to zero
I1 = I1_bot + I1_top; % should be equal to zero
I2 = I2_bot + I2_top;
I3 = I3_bot + I3_top;

y1 = y_loc(1); % m
y2 = y_loc(2); % m
y3 = y_loc(3); % m
y4 = y_loc(4); % m
B0 = -(y1*y2*y3 + y1*y2*y4 + y1*y3*y4 + y2*y3*y4); % m^3
B1 = y1*y2 + y1*y3 + y1*y4 + y2*y3 + y2*y4 + y3*y4; % m^2
B2 = -(y1+y2+y3+y4); % m
B3 = 1; % -
D0 = B0*I0 + B1*I1 + B2*I2 + B3*I3; % Pa*m^2
D1 = B1*I0 + B2*I1 + B3*I2; % Pa*m
D2 = B2*I0 + B3*I1; % Pa
D3 = B3*I0; % Pa/m

C0_short = -D0; % Pa*m^2
C1_short = -D1; % Pa*m

% Compute C0 and C1 (full method):
AA = zeros(2,2);
bb = zeros(2,1);
switch flag.int
    case 1 % inverse method: Gauss-Chebyshev quadrature
        F_bot_T = 1 ./ -sqrt( abs( (y_j_bot_T-y_loc(3)) .* (y_j_bot_T-y_loc(4)) ) );
        F_top_T = 1 ./  sqrt( abs( (y_j_top_T-y_loc(1)) .* (y_j_top_T-y_loc(2)) ) );        
        AA(1,1) = sum(F_bot_T .* w_j_bot_T); % 1/m
        AA(1,2) = sum(y_j_bot_T .* F_bot_T .* w_j_bot_T); % -
        AA(2,1) = sum(F_top_T .* w_j_top_T); % 1/m
        AA(2,2) = sum(y_j_top_T .* F_top_T .* w_j_top_T); % -
        bar_y_bot = (y_loc(2)-y_loc(1))/2; 
        bar_y_top = (y_loc(4)-y_loc(3))/2; 
        G_bot_U = -sigma_bot_U.C .* -ifs_Psi(y_j_bot_U,y_loc(3:4)) * bar_y_bot^2; % Pa*m^2
        G_top_U = -sigma_top_U.C .*  ifs_Psi(y_j_top_U,y_loc(1:2)) * bar_y_top^2; 
        help_bot_U = ( ( sum(G_bot_U .* w_j_bot_U ./ -(y_j_bot_U-y_i_bot_U')) )' + ...
                       ( sum(G_top_U .* w_j_top_U ./ -(y_j_top_U-y_i_bot_U')) )' ); % Pa*m^2
        help_top_U = ( ( sum(G_top_U .* w_j_top_U ./ -(y_j_top_U-y_i_top_U')) )' + ...
                       ( sum(G_bot_U .* w_j_bot_U ./ -(y_j_bot_U-y_i_top_U')) )' ); % Pa*m^2
%         help_bot_T = ifs_barylag([y_i_bot_U help_bot_U],y_i_bot_T);
%         help_top_T = ifs_barylag([y_i_top_U help_top_U],y_i_top_T);
%         bb(1) = -trapz(y_i_bot_T,(1 ./ Psi_1_bot_T) .* help_bot_T); % Pa*m
%         bb(2) = -trapz(y_i_top_T,(1 ./ Psi_1_top_T) .* help_top_T); 
        bb(1) = -trapz(y_i_bot_U,(1 ./ Psi_1_bot_U) .* help_bot_U); % Pa*m
        bb(2) = -trapz(y_i_top_U,(1 ./ Psi_1_top_U) .* help_top_U); 
    case 2 % inverse method: Matlab's variable-support trapezium rule
        AA(1,1) = trapz(y_j_bot_T, 1 ./ Psi_2_bot_T); % 1/m
        AA(1,2) = trapz(y_j_bot_T, y_j_bot_T ./ Psi_2_bot_T); % -
        AA(2,1) = trapz(y_j_top_T, 1 ./ Psi_2_top_T); % 1/m
        AA(2,2) = trapz(y_j_top_T, y_j_top_T ./ Psi_2_top_T); % -
        G_bot_U = -sigma_bot_U.C .* Psi_2_bot_U; % Pa*m^2
        G_top_U = -sigma_top_U.C .* Psi_2_top_U;       
        help_bot_U = ( ( trapz(y_j_bot_U, G_bot_U ./ -(y_j_bot_U-y_i_bot_U')) )' + ...
                       ( trapz(y_j_top_U, G_top_U ./ -(y_j_top_U-y_i_bot_U')) )' ); % Pa*m^2
        help_top_U = ( ( trapz(y_j_top_U, G_top_U ./ -(y_j_top_U-y_i_top_U')) )' + ...
                       ( trapz(y_j_bot_U, G_bot_U ./ -(y_j_bot_U-y_i_top_U')) )' ); 
%         help_bot_T = ifs_barylag([y_i_bot_U help_bot_U],y_i_bot_T);
%         help_top_T = ifs_barylag([y_i_top_U help_top_U],y_i_top_T);
%         bb(1) = -trapz(y_i_bot_T,(1 ./ Psi_1_bot_T) .* help_bot_T); % Pa*m
%         bb(2) = -trapz(y_i_top_T,(1 ./ Psi_1_top_T) .* help_top_T); 
        bb(1) = -trapz(y_i_bot_U,(1 ./ Psi_1_bot_U) .* help_bot_U); % Pa*m
        bb(2) = -trapz(y_i_top_U,(1 ./ Psi_1_top_U) .* help_top_U); 
end
cc = AA\bb;
C0 = cc(1); % Pa*m^2
C1 = cc(2); % Pa*m
% % % C0 = ( AA(2,2)*bb(1) - AA(1,2)*bb(2) ) / ( AA(1,1)*AA(2,2) - AA(1,2)*AA(2,1) ) ; % Pa*m^2
% % % C1 = 0;

% Check dimensions: A*c = b
% 1/m * Pa*m^2 + (-) * Pa*m = Pa*m
% OK

% % Scaling:
% L = y_bar_top - y_bar_bot; % distance between patch mid points, m
% C0 = C0/L^2;
% C1 = C1/L;
% C0_short = C0_short/L^2;
% C1_short = C1_short/L;
% D0 = D0/L^2;
% D1 = D1/L;
% D2 = D2;
% D3 = D3*L;
