function [delta,grad,y_delta] = ifs_delta(flag,model,numpar,plotpar,delta,p_inc,y_delta,y_loc)
%
% Computes the slip gradient "grad" and the slip "delta". Makes use of Gauss-Chebyshev
% integration or Matlab's variable-support trapezium rule with Chebyshev points, both on a
% staggered grid.     
%

global c_bot c_mer c_top

% Retrieve:
% a = model.a; 
A = model.A;
y_min = numpar.y_sim_min;
y_max = numpar.y_sim_max;
N_bot = numpar.N_bot;
N_top = numpar.N_top;
N_cheb = numpar.N_cheb;
N_mer = numpar.N_mer;

% Compute slip gradient:
switch flag.sim_type
    case {1,2} % uncoupled or coupled
        switch flag.int
            case {1,2} % inverse method: Gauss-Chebyshev quadrature, or Matlab's ...
                % variable-support trapezium rule
                [w_i_bot,w_j_bot,y_i_bot,y_j_bot] = ifs_GaussCheb(00,N_bot,y_loc(1),y_loc(2));
                [w_i_top,w_j_top,y_i_top,y_j_top] = ifs_GaussCheb(00,N_top,y_loc(3),y_loc(4));
            case 3 % forward method: discrete Chebyshev polynomials
                [~,~,~,y_j_bot] = ifs_GaussCheb(11,N_cheb/2,y_loc(1),y_loc(2));
                [~,~,~,y_j_top] = ifs_GaussCheb(11,N_cheb/2,y_loc(3),y_loc(4));
        end
    case 3 % merged
        switch flag.int
            case {1,2} % inverse method: Gauss-Chebyshev quadrature, or Matlab's ...
                % variable-support trapezium rule
                [w_i_mer,w_j_mer,y_i_mer,y_j_mer] = ifs_GaussCheb(00,N_mer,y_loc(1),y_loc(2));
            case 3 % forward method: discrete Chebyshev polynomials
                N_cheb = numpar.N_cheb;
                [~,~,~,y_j_mer] = ifs_GaussCheb(11,N_cheb/2,y_loc(1),y_loc(2));
        end
end

switch flag.sim_type
    case 1 % uncoupled
        sigma_bot = ifs_sigma(flag,model,numpar,delta,0,p_inc,y_delta,y_j_bot,y_j_bot);
        sigma_top = ifs_sigma(flag,model,numpar,delta,0,p_inc,y_delta,y_j_top,y_j_top);
        switch flag.int
            case 1 % inverse method: Gauss-Chebyshev quadrature
                Y_1_bot = (1/(pi^2*A)) * ifs_Psi(y_i_bot,y_loc(1:2));
                Y_1_top = (1/(pi^2*A)) * ifs_Psi(y_i_top,y_loc(3:4));
                Y_2_bot = -sigma_bot.C;
                Y_2_top = -sigma_top.C;
                grad_bot = Y_1_bot .* ( sum(Y_2_bot .* w_j_bot ./ -(y_j_bot-y_i_bot')) )';
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_bot(1) = 0; 
                    grad_bot(end) = 0;
                end
                grad_top = Y_1_top .* ( sum(Y_2_top .* w_j_top ./ -(y_j_top-y_i_top')) )';
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_top(1) = 0; 
                    grad_top(end) = 0;
                end
                delta_bot = cumsum(ifs_Psi(y_i_bot,y_loc(1:2)) .* grad_bot .* w_i_bot);
                delta_top = cumsum(ifs_Psi(y_i_top,y_loc(3:4)) .* grad_top .* w_i_top);
                y_delta_bot = y_i_bot;
                y_delta_top = y_i_top;
            case 2 % inverse method: Matlab's variable-support trapezium rule
                Y_1_bot = (1/(pi^2*A)) * ifs_Psi(y_i_bot,y_loc(1:2)); 
                Y_1_top = (1/(pi^2*A)) * ifs_Psi(y_i_top,y_loc(3:4));
                Y_2_bot =  -sigma_bot.C ./ ifs_Psi(y_j_bot,y_loc(1:2));
                Y_2_top =  -sigma_top.C ./ ifs_Psi(y_j_top,y_loc(3:4));
                grad_bot = Y_1_bot .* ( trapz(y_j_bot,Y_2_bot ./ -(y_j_bot-y_i_bot')) )';
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_bot(1) = 0; 
                    grad_bot(end) = 0;
                end
                grad_top = Y_1_top .* ( trapz(y_j_top,Y_2_top ./ -(y_j_top-y_i_top')) )';
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_top(1) = 0; 
                    grad_top(end) = 0;
                end
                delta_bot = cumtrapz(y_i_bot,grad_bot);
                delta_top = cumtrapz(y_i_top,grad_top);
                y_delta_bot = y_i_bot;
                y_delta_top = y_i_top;
            case 3 % forward method: discrete Chebyshev polynomials
                N = N_cheb/2;
                T = numpar.T_2;
                U = numpar.U_2;
                z_n = numpar.z_U_2;
                NN = diag(1./(1:N));
                Z = diag(sqrt(1-z_n.^2));
                Z_tilde = diag((sqrt(1-z_n.^2)).^(-1));
                bar_y_bot = (y_loc(2)-y_loc(1))/2;
                bar_y_top = (y_loc(4)-y_loc(3))/2;
                delta_bot = -Z*U*NN*c_bot*bar_y_bot;
                delta_top = -Z*U*NN*c_top*bar_y_top;
                grad_bot = Z_tilde*T(:,2:N+1)*c_bot;
                grad_top = Z_tilde*T(:,2:N+1)*c_top;
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_bot(1) = 0; 
                    grad_bot(end) = 0;
                    grad_top(1) = 0; 
                    grad_top(end) = 0;
                end
                y_delta_bot = y_j_bot;
                y_delta_top = y_j_top;
        end
        delta = [0;delta_bot;delta_top;0];
        grad = [0;grad_bot;grad_top;0];
        y_delta = [y_min;y_delta_bot;y_delta_top;y_max];
        
    case 2 % coupled
        sigma_bot = ifs_sigma(flag,model,numpar,delta,0,p_inc,y_delta,y_j_bot,y_j_bot);
        sigma_top = ifs_sigma(flag,model,numpar,delta,0,p_inc,y_delta,y_j_top,y_j_top);
        switch flag.int
            case 1 % inverse method: Gauss-Chebyshev quadrature
                Y_1_bot = (1/(pi^2*A)) * -ifs_Psi(y_i_bot,y_loc(1:4));
                Y_1_top = (1/(pi^2*A)) *  ifs_Psi(y_i_top,y_loc(1:4));
                Y_2_bot = -sigma_bot.C ./ -ifs_Psi(y_j_bot,y_loc(3:4));
                Y_2_top = -sigma_top.C ./  ifs_Psi(y_j_top,y_loc(1:2));
                grad_bot = Y_1_bot .* ...
                    ( ( sum(Y_2_bot .* w_j_bot ./ -(y_j_bot-y_i_bot')) )' + ...
                      ( sum(Y_2_top .* w_j_top ./ -(y_j_top-y_i_bot')) )' );
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_bot(1) = 0; 
                    grad_bot(end) = 0;
                end
                grad_top = Y_1_top .* ...
                    ( ( sum(Y_2_top .* w_j_top ./ -(y_j_top-y_i_top')) )' + ...
                      ( sum(Y_2_bot .* w_j_bot ./ -(y_j_bot-y_i_top')) )' );
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_top(1) = 0; 
                    grad_top(end) = 0;
                end
                delta_bot = ...
                    cumsum( ifs_Psi(y_i_bot,y_loc(1:2)) .* grad_bot .* w_i_bot);
                delta_top = ...
                    cumsum( ifs_Psi(y_i_top,y_loc(3:4)) .* grad_top .* w_i_top);
                y_delta_bot = y_i_bot;
                y_delta_top = y_i_top;
            case 2 % inverse method: Matlab's variable-support trapezium rule
                Y_1_bot = (1/(pi^2*A)) * -ifs_Psi(y_i_bot,y_loc(1:4));
                Y_1_top = (1/(pi^2*A)) *  ifs_Psi(y_i_top,y_loc(1:4));
                Y_2_bot =  -sigma_bot.C ./ -ifs_Psi(y_j_bot,y_loc(1:4));
                Y_2_top =  -sigma_top.C ./  ifs_Psi(y_j_top,y_loc(1:4));
                grad_bot = Y_1_bot .* ...
                    ( ( trapz(y_j_bot,Y_2_bot ./ -(y_j_bot-y_i_bot')) )' + ...
                      ( trapz(y_j_top,Y_2_top ./ -(y_j_top-y_i_bot')) )' );
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_bot(1) = 0; 
                    grad_bot(end) = 0;
                end
                grad_top = Y_1_top .* ...
                    ( ( trapz(y_j_top,Y_2_top ./ -(y_j_top-y_i_top')) )' + ...
                      ( trapz(y_j_bot,Y_2_bot ./ -(y_j_bot-y_i_top')) )' );
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_top(1) = 0; 
                    grad_top(end) = 0;
                end
                delta_bot = cumtrapz(y_i_bot,grad_bot);
                delta_top = cumtrapz(y_i_top,grad_top);
                y_delta_bot = y_i_bot;
                y_delta_top = y_i_top;
            case 3 % forward method: discrete Chebyshev polynomials
                N = N_cheb/2;
                T = numpar.T_2;
                U = numpar.U_2;
                z_n = numpar.z_U_2;
                NN = diag(1./(1:N));
                Z = diag(sqrt(1-z_n.^2));
                Z_tilde = diag((sqrt(1-z_n.^2)).^(-1));
                bar_y_bot = (y_loc(2)-y_loc(1))/2;
                bar_y_top = (y_loc(4)-y_loc(3))/2;
                delta_bot = -Z*U*NN*c_bot*bar_y_bot;
                delta_top = -Z*U*NN*c_top*bar_y_top;
                grad_bot = Z_tilde*T(:,2:N+1)*c_bot;
                grad_top = Z_tilde*T(:,2:N+1)*c_top;
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_bot(1) = 0; 
                    grad_bot(end) = 0;
                    grad_top(1) = 0; 
                    grad_top(end) = 0;
                end
                y_delta_bot = y_j_bot;
                y_delta_top = y_j_top;
        end
        delta = [0;delta_bot;delta_top;0];
        grad = [0;grad_bot;grad_top;0];
        y_delta = [y_min;y_delta_bot;y_delta_top;y_max];
        
    case 3 % merged
        sigma_mer = ifs_sigma(flag,model,numpar,delta,0,p_inc,y_delta,y_j_mer,y_j_mer);
        switch flag.int
            case 1 % inverse method: Gauss-Chebyshev quadrature
                Y_1_mer = (1/(pi^2*A)) * ifs_Psi(y_i_mer,y_loc(1:2));
                Y_2_mer = -sigma_mer.C;
                grad_mer = Y_1_mer .* ( sum(Y_2_mer .* w_j_mer ./ -(y_j_mer-y_i_mer')) )';
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_mer(1) = 0; 
                    grad_mer(end) = 0;
                end
                delta_mer = cumsum( sqrt( abs( (y_i_mer-y_loc(1)) .* (y_i_mer-y_loc(2)) ) ) ...
                    .* grad_mer .* w_i_mer);
                y_delta_mer = y_i_mer;
            case 2 % inverse method: Matlab's variable-support trapezium rule
                Y_1_mer = (1/(pi^2*A)) * ifs_Psi(y_i_mer,y_loc(1:2));
                Y_2_mer =  -sigma_mer.C ./ ifs_Psi(y_j_mer,y_loc(1:2));
                grad_mer = Y_1_mer .* ( trapz(y_j_mer,Y_2_mer ./ -(y_j_mer-y_i_mer')) )';
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_mer(1) = 0; 
                    grad_mer(end) = 0;
                end
                delta_mer = cumtrapz(y_i_mer,grad_mer);
                y_delta_mer = y_i_mer;
            case 3 % discrete Chebyshev polynomials; one interval per patch
                N = N_cheb/2;
                T = numpar.T_2;
                U = numpar.U_2;
                z_n = numpar.z_U_2;
                NN = diag(1./(1:N));
                Z = diag(sqrt(1-z_n.^2));
                Z_tilde = diag((sqrt(1-z_n.^2)).^(-1));
                bar_y_mer = (y_loc(2)-y_loc(1))/2;
                delta_mer = -Z*U*NN*c_mer*bar_y_mer;
                grad_mer = Z_tilde*T(:,2:N+1)*c_mer;
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_mer(1) = 0; 
                    grad_mer(end) = 0;
                end
                y_delta_mer = y_j_mer;
        end
        delta = [0;delta_mer;0];
        grad = [0;grad_mer;0];
        y_delta = [y_min;y_delta_mer;y_max];
end
