function [delta,grad,y_delta] = ifs_delta_sym(flag,model,numpar,plotpar,delta,p_inc,y_delta,...
    y_loc)
%
% Computes the slip gradient "grad" and the slip "delta". Makes use of Gauss-Chebyshev
% integration or Matlab's variable-support trapezium rule with Chebyshev points, both on a
% staggered grid. Makes use of symmetry around y = 0.    
%

% Retrieve:
G = model.G; % shear modulus, Pa
nu = model.nu; % Poisson's ratio, -
y_plot = plotpar.y_plot;
y_min = y_plot.min_sigma;
y_max = y_plot.max_sigma;

% Compute auxiliary variables:
A = G / (2*pi*(1-nu));

% Compute slip gradient:
switch flag.sim_type       
    case {1,2} % uncoupled or coupled
        N_top = numpar.N_top;
        [w_i_top,w_j_top,y_i_top,y_j_top] = ifs_GaussCheb(00,N_top,y_loc(3),y_loc(4));

    case 3 % merged
        N = numpar.N_mer;
        z_j = flip(cos( pi*((1:N)-0.5)/N ))';
        z_j_sym = z_j((N+1)/2:N); 
        z_i = flip(cos( pi*((1:N-1))/N ))';
        z_i_sym = z_i((N+1)/2:N-1); 
        bar_y_sym = y_loc;
        y_j_mer_sym = z_j_sym * bar_y_sym;
        y_i_mer_sym = z_i_sym * bar_y_sym;
        w_j_mer_sym = pi/N;
        w_i_mer_sym = pi/(N-1); 
        if delta ~= 0
            delta = delta(2:end-1);
            delta_sym = delta((N+1)/2:N); 
            y_delta = y_delta(2:end-1);
            y_delta_sym = y_delta((N+1)/2:N); 
        else
            delta_sym = 0;
            y_delta_sym = 0;
        end
end

switch flag.sim_type
    case 1 % uncoupled
        sigma_top = ifs_sigma(flag,model,numpar,delta,0,p_inc,y_delta,y_j_top,y_j_top);
        switch flag.int
            case 1 % Gauss-Chebyshev quadrature; one interval per patch
                Y_1_top = (1/(pi^2*A)) * ifs_Psi(y_i_top,y_loc(3:4));
                Y_2_top = -sigma_top.C;
                grad_top = Y_1_top .* ( sum(Y_2_top .* w_j_top ./ -(y_j_top-y_i_top')) )';
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_top(1) = 0; 
                    grad_top(end) = 0;
                end
                delta_top = ...
                    cumsum( ifs_Psi(y_i_top,y_loc(3:4)) .* grad_top .* w_i_top);
            case 2 % Matlab's variable-support trapezium rule; one interval per patch
                Y_1_top = (1/(pi^2*A)) * ifs_Psi(y_i_top,y_loc(3:4));
                Y_2_top =  -sigma_top.C ./ ifs_Psi(y_j_top,y_loc(3:4));
                grad_top = Y_1_top .* ( trapz(y_j_top,Y_2_top ./ -(y_j_top-y_i_top')) )';
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_top(1) = 0; 
                    grad_top(end) = 0;
                end
                delta_top = cumtrapz(y_i_top,grad_top);
        end
        y_delta_top = y_i_top;
        delta = [0;flip(delta_top);delta_top;0];
        grad = [0;-flip(grad_top);grad_top;0];
        y_delta = [y_min;-flip(y_delta_top);y_delta_top;y_max];
        
    case 2 % coupled
        sigma_top = ifs_sigma(flag,model,numpar,delta,0,p_inc,y_delta,y_j_top,y_j_top);
        switch flag.int
            case 1 % Gauss-Chebyshev quadrature; one interval per patch
                Y_1_top_sym = (1/(pi^2*A)) * ifs_Psi(y_i_top.^2,y_loc(1:2).^2);
                Y_2_top_sym = -sigma_top.C ./ ifs_Psi(y_j_top,y_loc(1:2));
                grad_top = Y_1_top_sym .* ( ( sum(2*y_j_top.*Y_2_top_sym .* w_j_top ./ ...
                    -(y_j_top.^2-y_i_top.^2')) )');
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_top(1) = 0; 
                    grad_top(end) = 0;
                end
                delta_top = cumsum( ifs_Psi(y_i_top,y_loc(3:4)) .* grad_top .* w_i_top);
            case 2 % Matlab's variable-support trapezium rule; one interval per patch
                Y_1_top_sym = (1/(pi^2*A)) * ifs_Psi(y_i_top.^2,y_loc(3:4).^2);
                Y_2_top_sym =  -sigma_top.C ./ ifs_Psi(y_j_top.^2,y_loc(3:4).^2);
                grad_top = Y_1_top_sym .* ( ( trapz(y_j_top,2*y_j_top.*Y_2_top_sym ./ ...
                    -(y_j_top.^2-y_i_top.^2')) )' );
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_top(1) = 0; 
                    grad_top(end) = 0;
                end
                delta_top = cumtrapz(y_i_top,grad_top);
        end
        y_delta_top = y_i_top;
        delta = [0;flip(delta_top);delta_top;0];
        grad = [0;-flip(grad_top);grad_top;0];
        y_delta = [y_min;-flip(y_delta_top);y_delta_top;y_max];
        
    case 3 % merged; because of symmetry, y_loc has only one element
        sigma_mer_sym = ifs_sigma(flag,model,numpar,delta_sym,0,p_inc,y_delta_sym,...
            y_j_mer_sym,y_j_mer_sym);
        switch flag.int
            case 1 % Gauss-Chebyshev quadrature; one interval per patch
                Y_1_mer_sym = (1/(pi^2*A)) * ifs_Psi(y_i_mer_sym.^2,y_loc^2);
                Y_2_mer_sym = -sigma_mer_sym.C;
                grad_mer_sym = 2*y_i_mer_sym.*Y_1_mer_sym .* ...
                    ( sum(Y_2_mer_sym .* w_j_mer_sym ./ -(y_j_mer_sym.^2-y_i_mer_sym.^2')) )';
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_mer_sym(end) = 0;
                end
                delta_mer_sym = cumsum( ifs_Psi(y_i_mer_sym.^2,y_loc^2) .* grad_mer_sym * ...
                    w_i_mer_sym);
                delta_mer_sym = delta_mer_sym - delta_mer_sym(end);
            case 2 % Matlab's variable-support trapezium rule; one interval per patch
                Y_1_mer_sym = (1/(pi^2*A)) * ifs_Psi(y_i_mer_sym.^2,y_loc^2);
                Y_2_mer_sym =  -sigma_mer_sym.C ./ ifs_Psi(y_j_mer_sym.^2,y_loc^2);
                grad_mer_sym = 2*y_i_mer_sym.*Y_1_mer_sym .* ...
                    ( trapz(y_j_mer_sym,Y_2_mer_sym ./ -(y_j_mer_sym.^2-y_i_mer_sym.^2')) )';
                if flag.zero == 1 % forces zero values on end points of slip gradient
                    grad_mer_sym(end) = 0;
                end
                delta_mer_sym = cumtrapz(y_i_mer_sym,grad_mer_sym);
                delta_mer_sym = delta_mer_sym - delta_mer_sym(end);
        end
        y_delta_mer_sym = y_i_mer_sym;
        delta = [0;flip(delta_mer_sym);delta_mer_sym;0];
        grad = [0;-flip(grad_mer_sym);grad_mer_sym;0];
        y_delta = [y_min;-flip(y_delta_mer_sym);y_delta_mer_sym;y_max];
end
