function [epsilon_tilde,sigma_tilde,u_tilde] = ifs_fido(flags,model,plotpar)
%
% Computes the scaled pre-slip strains epsilon_tilde_ij = epsilon_ij/p_inc and stresses
% sigma_tilde_ij = sigma_ij/p_inc, i = x,y, j = x,y, at specified constant values of x and y.
% For vertical faults only: also computes the pre-slip scaled displacements u_tilde_i =
% u_i/p_inc. This information is required to compare analytical solutions for an infinite
% domain with those of numerical simulations for a finite domain.      
%

% Retrieve variables:
a = model.a;
b = model.b;
c = model.c;
d = model.d;
alpha = model.alpha;
G = model.G;
nu = model.nu;
H = model.H; % height of simulation domain, m
W = model.W; % width of simulation domain, m
theta = model.theta;

n_fido = plotpar.n_fido; % nr. of points to plot strains and stresses at domain boundaries, -

% Initialize:
u_tilde = [];

% Compute auxiliary variables:
C_tilde = ( (1-2*nu)*alpha ) / ( 2*pi*(1-nu) );
D_tilde = C_tilde / G;
x_E =  W/2; % x values @ East boundary
x_W = -W/2; % x values @ West boundary
y_N =  H/2; % y values @ North boundary; note: should be above y = b
y_S = -H/2; % y values @ South boundary; note should be below y = -b
x_NS = linspace(x_W,x_E,n_fido); % x values @ North and South boundaries 
y_EW = linspace(y_S,y_N,n_fido); % y values @ East and West boundaries

% Compute G_xx at boundaries:
if flags.vert == 1 % vertical fault
    if flags.inf == 1
        G_xx_E = Green_vert_g_xx_inf(a,b,x_E,y_EW); % infinite reservoir
        G_xx_W = Green_vert_g_xx_inf(a,b,x_W,y_EW);
        G_xx_N = Green_vert_g_xx_inf(a,b,x_NS,y_N); 
        G_xx_S = Green_vert_g_xx_inf(a,b,x_NS,y_S); 
    else
        G_xx_E = Green_vert_g_xx(a,b,c,d,x_E,y_EW); % finite reservoir
        G_xx_W = Green_vert_g_xx(a,b,c,d,x_W,y_EW);
        G_xx_N = Green_vert_g_xx(a,b,c,d,x_NS,y_N); 
        G_xx_S = Green_vert_g_xx(a,b,c,d,x_NS,y_S); 
    end
else % inclined fault
    if flags.inf == 1
        G_xx_E = Green_incl_g_xx_inf(a,b,theta,x_E,y_EW); % infinite reservoir 
        G_xx_W = Green_incl_g_xx_inf(a,b,theta,x_W,y_EW);
        G_xx_N = Green_incl_g_xx_inf(a,b,theta,x_NS,y_N);
        G_xx_S = Green_incl_g_xx_inf(a,b,theta,x_NS,y_S); 
    else
        G_xx_E = Green_incl_g_xx(a,b,c,d,theta,x_E,y_EW); % finite reservoir 
        G_xx_W = Green_incl_g_xx(a,b,c,d,theta,x_W,y_EW);
        G_xx_N = Green_incl_g_xx(a,b,c,d,theta,x_NS,y_N);
        G_xx_S = Green_incl_g_xx(a,b,c,d,theta,x_NS,y_S); 
    end
end

% Compute G_yy at boundaries:
if flags.vert == 1 % vertical fault
    if flags.inf == 1
        G_yy_E = Green_vert_g_yy_inf(a,b,x_E,y_EW); % infinite reservoir
        G_yy_W = Green_vert_g_yy_inf(a,b,x_W,y_EW);
        G_yy_N = Green_vert_g_yy_inf(a,b,x_NS,y_N); 
        G_yy_S = Green_vert_g_yy_inf(a,b,x_NS,y_S); 
    else
        G_yy_E = Green_vert_g_yy(a,b,c,d,x_E,y_EW); % finite reservoir
        G_yy_W = Green_vert_g_yy(a,b,c,d,x_W,y_EW);
        G_yy_N = Green_vert_g_yy(a,b,c,d,x_NS,y_N); 
        G_yy_S = Green_vert_g_yy(a,b,c,d,x_NS,y_S); 
    end
else % inclined fault
    if flags.inf == 1
        G_yy_E = Green_incl_g_yy_inf(a,b,theta,x_E,y_EW); % infinite reservoir 
        G_yy_W = Green_incl_g_yy_inf(a,b,theta,x_W,y_EW);
        G_yy_N = Green_incl_g_yy_inf(a,b,theta,x_NS,y_N);
        G_yy_S = Green_incl_g_yy_inf(a,b,theta,x_NS,y_S); 
    else
        G_yy_E = Green_incl_g_yy(a,b,c,d,theta,x_E,y_EW); % finite reservoir 
        G_yy_W = Green_incl_g_yy(a,b,c,d,theta,x_W,y_EW);
        G_yy_N = Green_incl_g_yy(a,b,c,d,theta,x_NS,y_N);
        G_yy_S = Green_incl_g_yy(a,b,c,d,theta,x_NS,y_S); 
    end
end

% Compute G_xy at boundaries:
if flags.vert == 1 % vertical fault
    if flags.inf == 1
        G_xy_E = Green_vert_g_xy_inf(a,b,x_E,y_EW); % infinite reservoir
        G_xy_W = Green_vert_g_xy_inf(a,b,x_W,y_EW);
        G_xy_N = Green_vert_g_xy_inf(a,b,x_NS,y_N); 
        G_xy_S = Green_vert_g_xy_inf(a,b,x_NS,y_S); 
    else
        G_xy_E = Green_vert_g_xy(a,b,c,d,x_E,y_EW); % finite reservoir
        G_xy_W = Green_vert_g_xy(a,b,c,d,x_W,y_EW);
        G_xy_N = Green_vert_g_xy(a,b,c,d,x_NS,y_N); 
        G_xy_S = Green_vert_g_xy(a,b,c,d,x_NS,y_S); 
    end
else % inclined fault
    if flags.inf == 1
        G_xy_E = Green_incl_g_xy_inf(a,b,theta,x_E,y_EW); % infinite reservoir 
        G_xy_W = Green_incl_g_xy_inf(a,b,theta,x_W,y_EW);
        G_xy_N = Green_incl_g_xy_inf(a,b,theta,x_NS,y_N);
        G_xy_S = Green_incl_g_xy_inf(a,b,theta,x_NS,y_S); 
    else
        G_xy_E = Green_incl_g_xy(a,b,c,d,theta,x_E,y_EW); % finite reservoir 
        G_xy_W = Green_incl_g_xy(a,b,c,d,theta,x_W,y_EW);
        G_xy_N = Green_incl_g_xy(a,b,c,d,theta,x_NS,y_N);
        G_xy_S = Green_incl_g_xy(a,b,c,d,theta,x_NS,y_S); 
    end
end

% Compute G_x at boundaries:
if flags.vert == 1 % vertical fault
    if flags.inf == 1
        G_x_E = Green_vert_g_x_inf(a,b,x_E,y_EW); % infinite reservoir
        G_x_W = Green_vert_g_x_inf(a,b,x_W,y_EW);
        G_x_N = Green_vert_g_x_inf(a,b,x_NS,y_N); 
        G_x_S = Green_vert_g_x_inf(a,b,x_NS,y_S); 
    else
        G_x_E = Green_vert_g_x(a,b,c,d,x_E,y_EW); % finite reservoir
        G_x_W = Green_vert_g_x(a,b,c,d,x_W,y_EW);
        G_x_N = Green_vert_g_x(a,b,c,d,x_NS,y_N); 
        G_x_S = Green_vert_g_x(a,b,c,d,x_NS,y_S); 
    end
end

% Compute G_y at boundaries:
if flags.vert == 1 % vertical fault
    if flags.inf == 1
        G_y_E = Green_vert_g_y_inf(a,b,x_E,y_EW); % infinite reservoir
        G_y_W = Green_vert_g_y_inf(a,b,x_W,y_EW);
        G_y_N = Green_vert_g_y_inf(a,b,x_NS,y_N); 
        G_y_S = Green_vert_g_y_inf(a,b,x_NS,y_S); 
    else
        G_y_E = Green_vert_g_y(a,b,c,d,x_E,y_EW); % finite reservoir
        G_y_W = Green_vert_g_y(a,b,c,d,x_W,y_EW);
        G_y_N = Green_vert_g_y(a,b,c,d,x_NS,y_N); 
        G_y_S = Green_vert_g_y(a,b,c,d,x_NS,y_S); 
    end
end

% Compute scaled strains, stresses and displacements at boundaries:
epsilon_tilde.xx_E = D_tilde/2 * G_xx_E;
epsilon_tilde.xx_W = D_tilde/2 * G_xx_W;
epsilon_tilde.xx_N = D_tilde/2 * G_xx_N;
epsilon_tilde.xx_S = D_tilde/2 * G_xx_S;
epsilon_tilde.yy_E = D_tilde/2 * G_yy_E;
epsilon_tilde.yy_W = D_tilde/2 * G_yy_W;
epsilon_tilde.yy_N = D_tilde/2 * G_yy_N;
epsilon_tilde.yy_S = D_tilde/2 * G_yy_S;
epsilon_tilde.xy_E = D_tilde/2 * G_xy_E;
epsilon_tilde.xy_W = D_tilde/2 * G_xy_W;
epsilon_tilde.xy_N = D_tilde/2 * G_xy_N;
epsilon_tilde.xy_S = D_tilde/2 * G_xy_S;

sigma_tilde.xx_E = C_tilde * ( G_xx_E - 2*pi .* (-a < y_EW) .* (y_EW < b) .* (x_E <   d) );
sigma_tilde.xx_W = C_tilde * ( G_xx_W - 2*pi .* (-b < y_EW) .* (y_EW < a) .* ( -c < x_W) );
sigma_tilde.xx_N = C_tilde * G_xx_N;
sigma_tilde.xx_S = C_tilde * G_xx_S;
sigma_tilde.yy_E = C_tilde * ( G_yy_E - 2*pi .* (-a < y_EW) .* (y_EW < b) .* (x_E <   d) );
sigma_tilde.yy_W = C_tilde * ( G_yy_W - 2*pi .* (-b < y_EW) .* (y_EW < a) .* ( -c < x_W) );
sigma_tilde.yy_N = C_tilde * G_yy_N;
sigma_tilde.yy_S = C_tilde * G_yy_S;
sigma_tilde.xy_E = C_tilde * G_xy_E;
sigma_tilde.xy_W = C_tilde * G_xy_W;
sigma_tilde.xy_N = C_tilde * G_xy_N;
sigma_tilde.xy_S = C_tilde * G_xy_S;

if flags.vert == 1 % vertical fault
    u_tilde.x_E = D_tilde * G_x_E;
    u_tilde.x_W = D_tilde * G_x_W;
    u_tilde.x_N = D_tilde * G_x_N;
    u_tilde.x_S = D_tilde * G_x_S;
    u_tilde.y_E = D_tilde * G_y_E;
    u_tilde.y_W = D_tilde * G_y_W;
    u_tilde.y_N = D_tilde * G_y_N;
    u_tilde.y_S = D_tilde * G_y_S;
end