function data = ifs_frictionless(data,flag,model,numpar,plotpar) 
%
% Computes and plots depletion-induced stresses and fault slip along a frictionless vertical
% fault.  
%
% Only relevant for ifs_input_01_example_1. Figure used in Jansen and Meulenbroek (2022).
%

% Retrieve parameters:
a = model.a; % height of reservoir bottom part (for positive x values) or reservoir ...
    % top part (for negative x values), m; see Fig. 1 in Jansen et al. (2019)
b = model.b; % height of reservoir top part (for positive x values) or reservoir ...
    % bottom part (for negative x values), m
alpha = model.alpha; % Biot's coefficient, -
G = model.G; % shear modulus, Pa
nu = model.nu; % Poisson's ratio, -
n_pt_y = plotpar.y_plot.n_pt;
y_max = plotpar.y_plot.max_sigma;
y_min = plotpar.y_plot.min_sigma;
p_inc = plotpar.p_inc_ref;

% Set fixed parameters:
mu_st_save = model.mu_st; 
theta_save = model.theta; 
model.theta = pi/2; % temporarily reset to vertical fault
model.mu_st = 0; % temporarily reset to frictionless fault

% Compute auxiliary variables:
A = G / (2*pi*(1-nu));
C = ((1-2*nu)*alpha*p_inc) / (2*pi*(1-nu)); % scaling parameter, N/m^2

% Compute pre-slip Coulomb stresses:
[sigma,~,~,~,y_sigma] = ifs_sigma(flag,model,numpar,0,n_pt_y,p_inc,0,y_max,y_min);
sigma_C = sigma.C;
y_pre = ifs_y_pre(flag,model,numpar,p_inc);

% Compute slip:
y_delta = linspace(y_min,y_max,n_pt_y);
delta = ...
      0 * y_delta          .* ( y_delta <= -b       ) ...
    + (C/A) * (-y_delta-b) .* (-b        <   y_delta) .* (y_delta < -a) ...
    + (C/A) * (a-b)        .* (-a        <   y_delta) .* (y_delta <  a) ...
    + (C/A) * (y_delta-b)  .* ( a        <   y_delta) .* (y_delta <  b) ...
    + 0 * y_delta          .* ( b        <=  y_delta);
delta_BO = delta/(2*(1-nu)); % Bourne and Oates (2017) approximate solution

% Reset parameters:
model.mu_st= mu_st_save; 
model.theta = theta_save; 

% Save plot data:
data.frls.delta = delta; % fault slip, m
data.frls.delta_BO = delta_BO; % Bourne and Oates (2017) fault slip, m
data.frls.sigma_C = sigma_C; % pre-slip Coulomb stresses, Pa
data.frls.y_pre = y_pre; % pre-slip Coulomb stress zeros, m
data.frls.y_delta = y_delta; % y values for fault slip, m
data.frls.y_sigma = y_sigma; % y values for pre-slip Coulomb stresses, m

