function [flag,model,numpar,plotpar] = ifs_input_04_large_offset(flag)
%
% Input file for ifs_main.
%
% Example from paper Jansen and Meulenbroek(2025).
%
% Has an option for small and large offsets, i.e. small and  large fault throws relative to the
% reservoir height. 
%
% ---------------------------------------------------------------------------------------------
% Set input parameters:
% ---------------------------------------------------------------------------------------------
flag_large = 1; % local flag to indicate fault offset
    % 0 = small offset: t_f =  75 m; t_f/h = 0.3333; small-offset case 
    % 1 = large offset: t_f = 190 m; t_f/h = 0.8444; large-offset case 
model.h = 225; % reservoir height, m; see Fig. 1 in Jansen et al. (2019)
if flag_large == 0
    model.t_f = 75; % fault throw, m
else
    model.t_f = 190; % fault throw, m
end
model.c = inf; % reservoir width to the left of the fault, m
model.d = inf; % reservoir width to the right of the fault, m
model.delta_c = 0.02; % critical slip distance for slip weakening 0, m
model.D_center = 3500; % reservoir depth @ y = 0, m
model.alpha = 0.9; % Biot's coefficient, -
model.g = 9.81; % acceleration of gravity, m/s^2
model.G = 6500e6; % shear modulus, Pa
model.kappa = 0; % cohesion, -
model.K_eff = 0.5; % initial effective vertical/horizontal stress ratio, -
model.mu_dyn = 0.30; 
model.mu_st = 0.52; % static friction coefficient, - % % % pre-slip
model.nu = 0.15; % Poisson's ratio, -
model.p_0_center = 35e6; % initial pressure @ y = 0, Pa
model.phi = 0.15; % porosity, -
model.rho_f = 1020; % fluid density, kg/m^3
model.rho_s = 2650; % solid density, kg/m^3
model.sigma_v_0 = 0; % initial vertical stress @ y = 0, Pa. If set equal to zero, the ...
    % stress is computed from the weigth of the overburden; see function fault_slip_sigma.m.
model.theta = 70*pi/180; % dip angle, rad
model.H = 4500; % height of finite domain (only relevant if flag.fido == 1), m
model.W = 4500; % width of finite domain (only relevant if flag.fido == 1), m  

% ---------------------------------------------------------------------------------------------
% Set program flow flags and computational parameters:
% ---------------------------------------------------------------------------------------------
flag.cont = 1; % flag to force continuation in case of non-convergence of slip-weakening ...
    % simulation:
    % 0 = continuation disabled (for diagnostics) 
    % 1 = continuation enabled (recommended for regular use)
flag.coupled = 1; % coupled simulation
flag.depdep = 1; % flag to trigger computation of depth-dependent initial pressure and str.:  
    % 0 = depth-independent (for diagnostics)
    % 1 = depth-dependent (recommended for regular use)    
flag.echo = 2; % flag to trigger writing to screen of simulation progress:
    % 0 = no echo
    % 1 = limited echo; only info about main simulation components
    % 2 = full echo; info about each simulation step
flag.fido = 0; % flag to trigger computation of pre-slip stresses and displacements at the ...
    % boundaries of a finite domain to allow comparison with numerical simulations:
    % 0 = compensation switched off (for regular use)
    % 1 = compensation switched on (for diagnostics) 
flag.fric = 2; % slip-weakening friction
flag.int = 3; % forward method: discrete Chebyshev polynomials; only valid for delta < delta_c
flag.nuc = 4; % flag to select detection method for nucleation (onset of seismicity):
    % 3 = use the Uenishi and Rice criterion
    % 4 = use the patch-growth sign reversal criterion (only for flag.int == 3)
flag.N_cheb = 0; % flag to select number of Gauss-Chebyshev integration points:
    % 0 = use a fixed number of points for all values of depletion/injection
    % 1 = use an increasing number of points for increasing values of depletion/injection 
flag.reg = 1; % flag to control regularization of slip-dependent friction coefficient and ...
    % Coulomb stresses:
    % 0 = regularization switched off (for diagnostics)
    % 1 = regularization switched on (recommended for regular use)
flag.short = 0; % flag to trigger use of short cut procedure to compute coupling effect:
    % 0 = use full procedure (recommended for regular use)
    % 1 = use short cut procedure (for research purposes)
flag.store = 0; % flag to trigger storage of input data and results as binary files:
    % 0 = don't store data and results
    % 1 = store data and results
flag.varstep = 1; % flag to select variable pressure stepping:
    % 0 = fixed step size (for diagnostics)
    % 1 = variable step size (recommended for regular use)
flag.zero = 1; % flag to force zero values on end points of slip gradient:
    % 0 = forcing switched off (recommended for regular use)
    % 1 = forcing switched on (for diagnostics or cosmetics)        
    
% Computational parameters:
numpar.dpdt_PU = 30/50; % pressure drop per unit time (in practical units), MPa/yr
numpar.dp_dy_seis = 100; % threshold value of numerical derivative, Pa/m 
numpar.eps_reg = 0.00; % regularization parameter for slip-dependent friction coefficient, m
numpar.eta_reg = 0.10; % regularization parameter for Coulomb stresses, m
numpar.n_p_max = 5000; % maximum number of pressure steps; for memory allocation, -
numpar.N_cheb = 512; % fixed nr. of integr. pts. (over all patches) or Chebyshev polynomials, -
numpar.N_cheb_max = 512; % maximum variable number of integration points, -
numpar.N_cheb_min = 16; % minimum variable number of integration points, -
numpar.N_cheb_unit = 3; % average variable nr. of integration points per unit height, 1/m, -
numpar.p_max_bounds = -0.0e6; % highest incremental pressure (negative for depletion), Pa
numpar.p_min_bounds = -30.0e6; % lowest incremental pressure (negative for depletion), Pa
numpar.p_step_init = -0.1e6; % initial pressure step size (negative for depletion), Pa
numpar.p_step_check = 5; % maximum allowed relative growth of pressure step size, -
numpar.p_step_correct = 0.7; % correction factor for pressure step size, -, 
numpar.p_step_fix = -1e4; % fixed pressure step size (negative for depletion), Pa     
numpar.p_step_max = -1e6; % maximum allowed (largest absolute value) pressure step size ...
    % (negative for depletion), Pa
numpar.p_step_n_grow = 1.5; % exponent to determine agressiveness of pressure step udating, -
numpar.p_step_min = -1e-4;  % minimum allowed (smallest absolute value) pressure step size ...
    % (negative for depletion), Pa
numpar.y_step_max = 0.20;
numpar.y_step_target = 0.10;
numpar.n_iter_fric_max = 500;  % maximum allowed number of friction iterations, -
numpar.tol_fric = 1e-6; % relative convergence tolerance for friction iterations, -
numpar.f_damp_fric = 0.5; % damping factor for iterative updating of slip patch ...
    % boundaries; number between 0 and 1, -
numpar.small_patch = 0.05; % minimum patch size to trigger slip computation during pressure ...
    % stepping, m
numpar.tol_merge = 1; % distance between slip patches to trigger merging, m   
numpar.y_sim_min = -250; % minimum y value for simulation, m
numpar.y_sim_max =  250; % maximum y value for simulation, m

% ---------------------------------------------------------------------------------------------
% Set plotting flags and parameters:
% ---------------------------------------------------------------------------------------------
% Plotting flags:
flag.plot_cap = 0; % flag for plot captions (black) and plot file location captions (red) 
    % 0 = no captions
    % 1 = with captions
flag.plot_const_x = 0; % flag for plots of stresses for constant x values
    % 0 = plotting switched off
    % 1 = plotting switched on    
flag.plot_diagn = 1; % flag for diagnostic plots:
    % 0 = plotting switched off
    % 1 = plotting switched on
flag.plot_diagn_C = 0; % flag for diagnostic plotting of C0 and C1 (coupled simulation):
    % 0 = plotting switched off
    % 1 = plotting switched on
flag.plot_G_or_sigma = 1; % flag to determine display in image plots:
    % 0 = display dimensionless incremental total stresses (G_ij - 2 pi delta_Omega)
    % 1 = display pressure, and incremental total and effective stresses (sigma_ij)   
flag.plot_image = 0; % flag for image plots (stresses as a function of x and y):
    % 0 = plotting switched off
    % 1 = plotting switched on   
flag.plot_nota = 1; % flag to indicate notation in plot output:
    % 0 = old notation: Sigma_.. = sigma_..^0 + sigma_..
    % 1 = new notation: sigma = sigma^0 + Delta sigma; tau = tau^0 + Delta tau   
flag.plot_snap = 1; % flag for snapshot plots:
    % 0 = plotting switched off
    % 1 = plotting switched on  
flag.plot_adu = 0; % flag to trigger additional plotting of uncoupled slip patch bounds:
    % 0 = plotting switched off
    % 1 = plotting switched on  
flag.plot_amu = 0; % flag to trigger additional plotting of (incorrect) Mushkelishvili and
    % Weertman results: 
    % 0 = plotting switched off
    % 1 = plotting switched on  
flag.plot_asy = 0; % flag to trigger additional plotting of symmetric configuration results:
    % 0 = plotting switched off
    % 1 = plotting switched on  
if flag_large == 0
    switch flag.nuc
        case 3 % Uenishi & Rice criterion
            plotpar.p_inc_ref = -18.7721e6; % large-offset case, pre-slip, just before nucl.
        case 4 % patch-growth sign reversal criterion   
            plotpar.p_inc_ref = -18.7755e6; % small-offset case, pre-slip, just before nucleation
    end
else
    switch flag.nuc
        case 3 % Uenishi & Rice criterion
            plotpar.p_inc_ref = -11.6511e6; % large-offset case, pre-slip, just before nucl.
        case 4 % patch-growth sign reversal criterion
            plotpar.p_inc_ref = -11.6509e6; % large-offset case, pre-slip, just before nucl.
    end
end
if flag_large == 0
    switch flag.nuc
        case 3 % Uenishi & Rice criterion
            plotpar.p_inc_snap = -18.7721e6; % small-offset case, pre-slip, just before nucl.
        case 4 % patch-growth sign reversal criterion
            plotpar.p_inc_snap = -18.7755e6; % small-offset case, pre-slip, just before nucl.
    end
else
    switch flag.nuc
        case 3 % Uenishi & Rice criterion
            plotpar.p_inc_snap = -11.6509e6; % large-offset case, pre-slip, just before nucl.
        case 4 % patch-growth sign reversal criterion
            plotpar.p_inc_snap = -11.6511e6; % large-offset case, pre-slip, just before nucl.
    end
end

% Minimum and maximum x and y values for various plots:
if flag_large == 0
    plotpar.x_plot.min_delta         = -0e-3; % m % % % small offset
    plotpar.x_plot.max_delta         =  6e-3;
else
    plotpar.x_plot.min_delta         = -0e-3; % m % % % large offset
    plotpar.x_plot.max_delta         =  2.5e-3;
end
plotpar.x_plot.min_delta_1       = -0.5e-3; % m
plotpar.x_plot.max_delta_1       =  3.5e-3; % m
plotpar.x_plot.min_delta_2       =     0; % mm
plotpar.x_plot.max_delta_2       =   3.5; % mm
plotpar.x_plot.min_gr_delta      =  -2.5; % mm/m
plotpar.x_plot.max_gr_delta      =   2.5; % mm/m
plotpar.x_plot.min_sigma         =    -3; % MPa
plotpar.x_plot.max_sigma         =    27; % MPa
plotpar.x_plot.min_sigma_1       =    -2; % detailed view, MPa
plotpar.x_plot.max_sigma_1       =     2; % detailed view, MPa
plotpar.x_plot.min_sigma_C       =   -15; % MPa % % % pre-slip
plotpar.x_plot.max_sigma_C       =     5; % MPa % % % pre-slip
plotpar.x_plot.min_sigma_C_1     =    -12; % MPa
plotpar.x_plot.max_sigma_C_1     =     12; % MPa
if flag_large == 0 % small-offset case
    plotpar.x_plot.min_bounds    =    13; % MPa 
    plotpar.x_plot.max_bounds    =    19; % MPa 
else % large-offset case
    plotpar.x_plot.min_bounds    =     9; % MPa 
    plotpar.x_plot.max_bounds    =    12; % MPa 
end
plotpar.x_plot_min_bounds_detail =    25; % MPa
plotpar.x_plot_max_bounds_detail =    29; % MPa
if flag_large == 0 % small-offset case
    plotpar.x_plot_min_nucl      =    16.0; % nucleation plot, MPa 
    plotpar.x_plot_max_nucl      =    19.0; %    "        "  , MPa 
else % large-offset case 
    plotpar.x_plot_min_nucl      =    10.5; % nucleation plot, MPa 
    plotpar.x_plot_max_nucl      =    12.0; %    "        "  , MPa 
end
plotpar.y_plot.min_delta         =  -250; % covers both slip patches (overview), m
plotpar.y_plot.max_delta         =   250; % covers both slip patches (overview), m
if flag_large == 0 % small-offset case
    plotpar.y_plot.min_delta_1   =  -250; % covers both slip patches (in detail), m 
    plotpar.y_plot.max_delta_1   =   250; % covers both slip patches (in detail), m 
else % large-offset case
    plotpar.y_plot.min_delta_1   =   -30; % covers both slip patches (in detail), m 
    plotpar.y_plot.max_delta_1   =    30; % covers both slip patches (in detail), m 
end
plotpar.y_plot.min_delta_2       =    60; % covers top patch only (further detail), m
plotpar.y_plot.max_delta_2       =    80; % covers top patch only (further detail), m
plotpar.y_plot.min_sigma         =  -250; % covers both slip patches (overview), m
plotpar.y_plot.max_sigma         =   250; % covers both slip patches (overview), m
if flag_large == 0 % small-offset case
    plotpar.y_plot.min_sigma_1   =  -250; % covers both slip patches (in detail), m
    plotpar.y_plot.max_sigma_1   =   250; % covers both slip patches (in detail), m
else % large-offset case
    plotpar.y_plot.min_sigma_1   =   -30; % covers both slip patches (in detail), m
    plotpar.y_plot.max_sigma_1   =    30; % covers both slip patches (in detail), m
end
if flag_large == 0 % small-offset case
    plotpar.y_plot.min_bounds    =  -100; % covers both slip patches (in detail), m
    plotpar.y_plot.max_bounds    =   100; % covers both slip patches (in detail), m
else % large-offset case
    plotpar.y_plot.min_bounds    =   -30; % covers both slip patches (in detail), m
    plotpar.y_plot.max_bounds    =    30; % covers both slip patches (in detail), m
end
plotpar.y_plot_min_bounds_detail =   -20; % m
plotpar.y_plot_max_bounds_detail =    20; % m
plotpar.y_plot_max_d_sigma_C_dy  =    76; % m
plotpar.y_plot_min_d_sigma_C_dy  =    74; % m
plotpar.y_plot_min_nucl          =    10; % nucleation plot, MPa 
plotpar.y_plot_max_nucl          =    25; %     "        " , MPa
plotpar.y_plot.n_pt              =   500; % number of points between y_plot.min_... and ...
                                          % y_plot.max_... % (only relevant for some plots) 
    
% Colors for line plots:
plotpar.colors.black   = [0/255 0/255 0/255];
plotpar.colors.blue   = [0/255 128/255 255/255];
plotpar.colors.coal   = [128/255 128/255 128/255];
plotpar.colors.gray   = [235/255 235/255 235/255];
plotpar.colors.green  = [50/255 168/255 82/255];
plotpar.colors.orange = [255/255 185/255 0/255];
plotpar.colors.purple = [204/255 0/255 204/255];
plotpar.colors.red    = [255/255 0/255 0/255];
plotpar.colors.white  = [255/255 255/255 255/255];

% ---------------------------------------------------------------------------------------------
% End of input.
% ---------------------------------------------------------------------------------------------
