% ifs_main_eigen_UandR
%
% Computes the nucleation length for a single patch following the approach of Uenishi and Rice
% (2003), and a symmetric double patch following the approach of Jansen and Meulenbroek (2025).  
%

clear all
close all

black  = [0/255 0/255 0/255];
blue   = [0/255 128/255 255/255];
coal   = [128/255 128/255 128/255];
gray   = [235/255 235/255 235/255];
green  = [50/255 168/255 82/255];
orange = [255/255 185/255 0/255];
purple = [204/255 0/255 204/255];
red    = [255/255 0/255 0/255];

% Single-patch solution for constant w_tilde:
N = 32; % number of Chebyshev poynomials
[U,z_n] = ifs_cheb_matrix_U(N,N,2);
NN = diag(1./(1:N));
Z = diag(sqrt(1-z_n.^2));
B = U\Z*U*NN;
% B = 2*U'*Z^3*U*NN / (N+1);
[~,lambda_tilde] = eigs(B,1);
format long
f = 1/lambda_tilde % result for N Chebyshev poynomials  
f_half = f/2
format short

% Saved results for an increasing number of polynomials:
j_tab = 1:10;
N_tab = 2.^j_tab';
f_tab = [
1.154700538379251; % N = 2
1.156733729329586;
1.157671718468133;
1.157765705964295;
1.157773302582817;
1.157773844957783;
1.157773881197478;
1.157773883538827;
1.157773883687589;
1.157773883696961]; % N = 2^10 = 1024
figure % Figure 1
semilogx(N_tab,f_tab,'o','MarkerFaceColor','red') % convergence plot for f
box on
grid on
xlabel('$N$','Interpreter','latex','FontSize',14)
ylabel('$f$','Interpreter','latex','FontSize',14)

f_save = f_tab(end);
lambda_save = 1/f_save;

% Symmetric double-patch solution for w_tilde = 0:
%N = 32; % number of Chebyshev poynomials
N = 2^11; % number of Chebyshev poynomials
[U,z_n] = ifs_cheb_matrix_U(N,N,2);
NN = diag(1./(1:N));
Z = diag(sqrt(1-z_n.^2));
epsilon = 1e-3;
alpha = 2 + epsilon + z_n;
omega = -alpha + sqrt(alpha.^2-1);
help = omega.^(1:N);
Omega = help./sqrt(alpha.^2-1);
B_hat = (U + Omega)\(Z*U*NN);
[~,lambda_hat_min] = eigs(B_hat,1);
format long
f_min = 1/lambda_hat_min % result for N Chebyshev poynomials  
format short

% Saved results for an increasing number of polynomials for w_tilde = 0:
j_tab = 1:14;
N_tab = 2.^j_tab';
z_p_tab = 1+(cos( pi*(N_tab)./(N_tab+1) ));
f_min_tab = [
0.885580997830329; % N = 2
0.836463322900810;
0.791483826401930;
0.756826969798595;
0.730676479635154;
0.710729498018508;
0.695203508316613;
0.682854118208916;
0.672830692480168;
0.664547714491617;
0.657595081738048;
0.651679523493524;
0.646586854484439;
0.642157487948229]; % N = 2^14 = 16384
figure % Figure 2
semilogx(N_tab,f_min_tab,'o','MarkerFaceColor','red') % convergence plot for f
box on
grid on
xlabel('$N$','Interpreter','latex','FontSize',14)
ylabel('$f(\hat r = 0)$','Interpreter','latex','FontSize',14)

%clear all

% Symmetric double-patch solution for constant w_tilde:
N = 512; % number of Chebyshev polynomials
N_r = 200; % number of discretization points for r_hat
r_hat_min = 0;
r_hat_max = 5;
[U,z_n] = ifs_cheb_matrix_U(N,N,2);
[T_help,zeta_m] = ifs_cheb_matrix_T(N+1,N+1,1);
T = T_help(:,2:N+1); % T is an N+1 x N matrix 
z_p = flip(cos( pi*(1:N)/(N+1) ))'; % auxiliary first-kind Chebyshev points
NN = diag(1./(1:N));
Z = diag(sqrt(1-z_n.^2));
r_hat = linspace(r_hat_min,r_hat_max,N_r);
r_bar = 2 * (r_hat + 1);
lambda_hat_ana = zeros(1,N_r);
lambda_hat_num = zeros(1,N_r);
tic
for i_r = 1:N_r
    Z_hat = (1/(N+1))*(2*zeta_m'+r_bar(i_r))./((zeta_m'-z_p).*(zeta_m'+z_p+r_bar(i_r)));
    B_hat = (Z_hat*T)\(Z*U*NN);
    [~,lambda_hat_num(i_r)] = eigs(B_hat,1);
end
toc
tic
for i_r = 1:N_r
    alpha = r_bar(i_r)+z_p;
    omega = -alpha + sqrt(alpha.^2-1);
    help = omega.^(1:N);
    Omega = help./sqrt(alpha.^2-1);
    B_hat = (U + Omega)\(Z*U*NN);
    [~,lambda_hat_ana(i_r)] = eigs(B_hat,1);
end
toc
f_hat_num = 1./lambda_hat_num;
f_hat_ana = 1./lambda_hat_ana;
% format long
% 1/lambda_hat
% format short

figure % Figure 3
plot(r_hat,ones(1,N_r)*f_save,'Color',red,'LineWidth',1.5)
hold on
plot(r_hat,f_hat_ana,'Color',blue,'LineWidth',1.5)
plot(r_hat,f_hat_num,'--','Color',orange,'LineWidth',1.5)
hold off
xlabel('$\hat r$','Interpreter','latex','FontSize',14)
ylabel('$f(\hat r)$','Interpreter','latex','FontSize',14)
legend('Single patch; ${f = 1.1577}$','Symmetric coupled patches (analytical)', ...
    'Symmetric coupled patches (numerical)','Interpreter','latex','FontSize',14,'Location','SouthEast')
axis([r_hat_min,r_hat_max,0.8,1.2])
box on
grid on

figure % Figure 4
plot(r_hat,ones(1,N_r)*lambda_save,'Color',red,'LineWidth',1.5)
hold on
plot(r_hat,lambda_hat_ana,'Color',blue,'LineWidth',1.5)
plot(r_hat,lambda_hat_num,'--','Color',orange,'LineWidth',1.5)
hold off
xlabel('$\hat r$','Interpreter','latex','FontSize',14)
ylabel('$\lambda_{max}(\hat r)$','Interpreter','latex','FontSize',14)
legend('Single patch; ${\lambda_{max} = 1/1.1577}$','Symmetric coupled patches (analytical)',...
    'Symmetric coupled patches (numerical)','Interpreter','latex','FontSize',14,'Location','NorthEast')
axis([r_hat_min,r_hat_max,0.8,1.2])
box on
grid on

% Fit with Matlab routine lsqcurvefit (Optimization Toolbox):
F =  @(x,xdata)0.579 *(1-(1/pi)*asec(x(1)*r_hat.^(x(2))+1)).^(-1);
x0 = [0.8 16.5];
[x,resnorm,~,exitflag,output] = lsqcurvefit(F,x0,r_hat,f_hat_ana)
figure % Figure 5
plot(r_hat,ones(1,N_r)*f_save,'Color',red,'LineWidth',1.5)
hold on
plot(r_hat,f_hat_ana,'Color',blue,'LineWidth',1.5)
plot(r_hat,f_hat_num,'--','Color',orange,'LineWidth',1.5)
plot(r_hat,F(x,r_hat),'-.','Color',green,'LineWidth',1.5)
hold off
xlabel('$\hat r$','Interpreter','latex','FontSize',14)
ylabel('$f(\hat r)$','Interpreter','latex','FontSize',14)
legend('Single patch; ${f = 1.158}$','Symmetric coupled patches (analytical)', ...
    '\hspace{1.9 cm} " \hspace{1.9 cm} (numerical)','\hspace{1.9 cm} " \hspace{1.9 cm} (fitted)',...
    'Interpreter','latex','FontSize',14,'Location','SouthEast')
axis([r_hat_min,r_hat_max,0.8,1.2])
box on
grid on

% Print table: 
fprintf('%7.4f,%7.4f\r',r_hat,f_hat_ana);
