% Script file ifs_main_nucleation
%
% Dedicated version of ifs_main for use with input file ifs_input_06_nucleation.m to create
% plots of nucleation pressure/length and aseismic moments versus dimensionless fault offset
% t_f/h. Most of the simulation parameters are set in the input file, but some of them are
% overwritten in this file, depending on which of the computation options is selected:
% 1) Computation/plotting of nucleation/merging pressure/length and aseismic moment (uncoupled) 
% 2) Computation/plotting of nucleation/merging pressure/length and aseismic moment (coupled) 
% Both options are both performed by forward simulation with slip-weakening friction until
% nucleation or merging occurs. 
%

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Set input parameters:
% ---------------------------------------------------------------------------------------------

flag_from_file = 1; % local flag to select computation or reading from file
    % 0 = compute new data (time consuming)
    % 1 = read data from file (e.g. to modify plot layout)

flag_comp = 2; % local flag to select computation
    % 1 = uncoupled
    % 2 = coupled

flag_echo = 1; % local flag to trigger writing to screen of simulation progress
    % 0 = no echo
    % 1 = limited echo; only info about main simulation components
    % 2 = full echo; info about each simulation step

% Assign run identifier (to store/read results):
switch flag_comp
    case 1
        admin.run_ID = 'Nucleation_plot_base_case_uncoupled';
    case 2
        admin.run_ID = 'Nucleation_plot_base_case_coupled';
end

% ---------------------------------------------------------------------------------------------
% End of input.
% ---------------------------------------------------------------------------------------------

% Set input file:
admin.input = 06; % ifs_input_06_nucleation.m.

% Set paths for input and output:
admin.path_out  = ...
    'C:\Users\djansen\My Documents\Onderzoek\Induced seismicity\IFS\IFS output files\';
file_name_out = ['ifs_output_',admin.run_ID];
if flag_comp == 3
    file_name_in  = ['ifs_output_',input_run_ID];
end

if flag_from_file == 0 % Compute new data:
    % Write to screen:
    fprintf('\r*** Induced Fault Slip (IFS) - Nucleation plot ***\r');

    % Initialize:
    [data,flag,numpar] = ifs_initialize;

    % Read input:
    [flag,model,numpar,plotpar] = ifs_read_input(admin,flag,numpar);

    % Write to screen:
    fprintf('\rIFS run ID: %s\r', admin.run_ID);
    fprintf('\rIFS input file nr.: %02d\r', admin.input);

    % Set variables:
    N = 45; % total number of fault offset steps
    N_nonuc = 4; % number of steps with nucleation pressure below largest depletion
    Delta_t_f = 5; % fault offset increment, - 
    Delta_y_mer_vec = zeros(1,N);
    Delta_y_eig_vec = zeros(1,N);
    Delta_y_sign_vec = zeros(1,N);
    Delta_y_UR_vec = zeros(1,N);
    j_merge_vec = zeros(1,N);
    j_seismic_vec = zeros(1,N);
    delta_seismic_max_vec = zeros(1,N); % maximum slip at nucleation, m
    m_vec = zeros(1,N); % seismic moment per unit strike length, Nm/m
    m_star_vec = zeros(1,N); % aseismic moment moment per unit strike length, Nm/m
    p_min_vec = zeros(1,N);
    p_merge_vec = zeros(1,N);
    p_seismic_vec = zeros(1,N);
 
    % Compute data for increasing values of fault throw t_f:
    t_f_vec = (0:N-1) * Delta_t_f;
    switch flag_comp
        case 1 % uncoupled
            N_start = 2; 
            flag.coupled = 0; % decoupled simulation
            flag.fric = 2; % slip-weakening friction
            model.mu_st = 0.52; % static friction coefficient, -
            flag.int = 3; % forward method: discrete Chebyshev polynomials
        case 2 % coupled
            N_start = 2; 
            flag.coupled = 1; % coupled simulation
            flag.fric = 2; % slip-weakening friction
            model.mu_st = 0.52; % static friction coefficient, -
            flag.int = 3; % forward method: discrete Chebyshev polynomials
    end
    for i = N_start:N
        flag.echo = flag_echo;
        model.t_f = t_f_vec(i);
        fprintf('\rFault throw t_f = %g m\r',t_f_vec(i));
        [flag,numpar,plotpar] = ifs_check(admin,flag,model,numpar,plotpar); 
        [data,flag,model,numpar,plotpar] = ifs_auxiliary(admin,data,flag,model,numpar,plotpar);
        [data,flag,model,numpar] = ifs_patch_bounds(data,flag,model,numpar,plotpar);
        j_merge_vec(i) = data.j_merge_y_til; % pressure step at which slip patches have merged, -
        j_seismic_vec(i) = data.j_seismic; % pressure step at which seismicity occurs, Pa
        if j_seismic_vec(i) == 0 && i > N_nonuc
            p_merge_vec(i) = data.p_inc(j_merge_vec(i)); % depletion pressure at which merging occurs, Pa
        end
        p_seismic_vec(i) = data.p_seismic(end); % depletion pressure at which seismicity occurs, Pa
        m_star_vec(i) = data.m_star; % aseismic moment per unit strike length at nucleation, Nm/m
        delta_seismic_max_vec(i) = max(data.delta_seismic); % maximum slip at nucelation, m
        Delta_y_mer_vec(i) = data.Delta_y_mer_top(end); % top patch length at moment of merging, m
        Delta_y_eig_vec(i) = data.Delta_y_eig_top(end); % nucleation length top patch (eigenvalue-based), m
        Delta_y_sign_vec(i) = data.Delta_y_sign_top(end); % nucleation length top patch (sign reversal), m
        Delta_y_UR_vec(i) = data.Delta_y_UR_top(end); % nucleation length top patch (Uenishi & Rice), m
    end

    % Store input data and results as binary files:
    localdata.delta_seismic_max_vec = delta_seismic_max_vec;
    localdata.m_vec = m_vec;
    localdata.m_star_vec = m_star_vec;
    localdata.N_nonuc = N_nonuc;
    localdata.t_f_vec = t_f_vec; 
    localdata.p_merge_vec = p_merge_vec; 
    localdata.p_seismic_vec = p_seismic_vec; 
    localdata.Delta_y_mer_vec = Delta_y_mer_vec; 
    localdata.Delta_y_eig_vec = Delta_y_eig_vec; 
    localdata.Delta_y_sign_vec = Delta_y_sign_vec; 
    localdata.Delta_y_UR_vec = Delta_y_UR_vec;
    file_name_out = ['ifs_output_',admin.run_ID];
    save([admin.path_out,file_name_out],'data','flag','localdata','model','numpar','plotpar');
    fprintf('\nBinary output stored in: ifs_output_%s.mat\n',admin.run_ID);

else % Read data from file:
    load([admin.path_out,file_name_out],'data','flag','localdata','model','numpar','plotpar');
    delta_seismic_max_vec = localdata.delta_seismic_max_vec;
    m_vec = localdata.m_vec;
    m_star_vec = localdata.m_star_vec;
    N_nonuc = localdata.N_nonuc;
    t_f_vec = localdata.t_f_vec; 
    p_merge_vec = localdata.p_merge_vec; 
    p_seismic_vec = localdata.p_seismic_vec;
    Delta_y_mer_vec = localdata.Delta_y_mer_vec; 
    Delta_y_eig_vec = localdata.Delta_y_eig_vec; 
    Delta_y_sign_vec = localdata.Delta_y_sign_vec; 
    Delta_y_UR_vec = localdata.Delta_y_UR_vec; 
end

% Retrieve data:
black = plotpar.colors.black;
blue = plotpar.colors.blue;
coal = plotpar.colors.coal;
green = plotpar.colors.green;
orange = plotpar.colors.orange;
purple = plotpar.colors.purple;
red = plotpar.colors.red;
white = plotpar.colors.white;

% Create plots:
[~,j] = find(p_seismic_vec);
N_seis = j(end);
t_f_max_plot = t_f_vec(1:N_nonuc); % first N_nonuc steps don't reach nucleation
t_f_seis_plot = t_f_vec(N_nonuc+1:N_seis);
t_f_merge_plot = t_f_vec(N_seis+1:end);
p_max_plot = ones(N_nonuc,1)*-30e6; % first N_nonuc steps don't reach nucleation
p_seismic_plot = p_seismic_vec(N_nonuc+1:N_seis);
p_merge_plot = p_merge_vec(N_seis+1:end);
Delta_y_eig_plot = Delta_y_eig_vec(N_nonuc+1:N_seis);
Delta_y_sign_plot = Delta_y_sign_vec(N_nonuc+1:N_seis);
Delta_y_UR_plot = Delta_y_UR_vec(N_nonuc+1:N_seis);
Delta_y_mer_plot = Delta_y_mer_vec(N_seis+1:end);
delta_seismic_max_plot = delta_seismic_max_vec;
m_star_plot = m_star_vec;
t_tilde_f_plot = t_f_vec/225;
t_tilde_f_max_plot = t_f_max_plot/225;
t_tilde_f_seis_plot = t_f_seis_plot/225;
t_tilde_f_merge_plot = t_f_merge_plot/225;

figure
subplot(2,1,1)
line([ 0.33 0.33],[0 2],'LineStyle','-.','Color',coal,'LineWidth',0.75)
hold on
line([ 0.33 0.33],[15 30],'LineStyle','-.','Color',coal,'LineWidth',0.75)
line([ 0.84 0.84],[0 30],'LineStyle','-.','Color',coal,'LineWidth',0.75)
L1 = plot(t_tilde_f_max_plot,-p_max_plot/1e6,'o','MarkerSize',5,'MarkerEdgeColor',red,...
    'MarkerFaceColor',white);
L2 = plot(t_tilde_f_seis_plot,-p_seismic_plot/1e6,'o','MarkerSize',5,'MarkerEdgeColor',red,...
    'MarkerFaceColor',red);
L3 = plot(t_tilde_f_merge_plot(1:end-2),-p_merge_plot(1:end-2)/1e6,'o',...
    'MarkerSize',5,'MarkerEdgeColor',red,'MarkerFaceColor',blue);
L4 = plot(t_tilde_f_merge_plot(end-2:end),-p_merge_plot(end-2:end)/1e6,'o',...
    'MarkerSize',5,'MarkerEdgeColor',blue,'MarkerFaceColor',blue);
hold off
xlabel('$\tilde t_f$ (-)','Interpreter','latex','FontSize',11)
ylabel('$-\Delta p^*$ (MPa)','Interpreter','latex','FontSize',11)
axis([0 1 0 30])
box on
L1 = legend([L1,L2,L3,L4],{...
    '$No \; nucleation \; or \; merging$',...
    '$Nucleation \; before \; merging$',...
    '$Merging \; before \; nucleation$',...
    '$Merging \; without \; nucleation$'},...
    'Interpreter','latex','FontSize',9,'Location','SouthWest');
set(L1,'Box','off')

subplot(2,1,2)
line([ 0.33 0.33],[0 2],'LineStyle','-.','Color',coal,'LineWidth',0.75)
hold on
line([ 0.33 0.33],[15 25],'LineStyle','-.','Color',coal,'LineWidth',0.75)
line([ 0.84 0.84],[0 25],'LineStyle','-.','Color',coal,'LineWidth',0.75)
L3 = plot(t_tilde_f_seis_plot,Delta_y_UR_plot,'o','MarkerSize',5,'MarkerEdgeColor',green,...
    'MarkerFaceColor',green);
L2 = plot(t_tilde_f_seis_plot,Delta_y_eig_plot,'^','MarkerSize',6,'MarkerEdgeColor',blue,...
    'MarkerFaceColor',blue);
L1 = plot(t_tilde_f_seis_plot,Delta_y_sign_plot,'o','MarkerSize',5,'MarkerEdgeColor',red,...
    'MarkerFaceColor',red);
L4 = plot(t_tilde_f_merge_plot(1:end-2),Delta_y_mer_plot(1:end-2),'o','MarkerSize',5,...
    'MarkerEdgeColor',red,'MarkerFaceColor',blue);
L5 = plot(t_tilde_f_merge_plot(end-2:end),Delta_y_mer_plot(end-2:end),'o','MarkerSize',5,...
    'MarkerEdgeColor',blue,'MarkerFaceColor',blue);
hold off
xlabel('$\tilde t_f$ (-)','Interpreter','latex','FontSize',11)
ylabel('$\Delta y^*$ (m)','Interpreter','latex','FontSize',11)
axis([0 1 0 25])
box on
L2 = legend([L1,L2,L3,L4,L5],{ ...
    '$Nucleation \; before \; merging \; (Simulation)$', ...
    '$Nucleation \; before \; merging \; (Eigen value)$', ...
    '$Nucleation \; before \; merging \; (EU\&R)$', ...
    '$Merging \; before \; nucleation$',...
    '$Merging \; without \; nucleation$'},...
    'Interpreter','latex','FontSize',9,'Location','SouthWest');
set(L2,'Box','off')

figure
subplot(2,1,1)
line([ 0.33 0.33],[0 8e8],'LineStyle','-.','Color',coal,'LineWidth',0.75)
hold on
line([ 0.84 0.84],[0 8e8],'LineStyle','-.','Color',coal,'LineWidth',0.75)
plot(t_tilde_f_plot,m_star_plot,'o','MarkerSize',5,'MarkerEdgeColor',blue,...
    'MarkerFaceColor',blue)
hold off
xlabel('$\tilde t_f$ (-)','Interpreter','latex','FontSize',11)
ylabel('$m^*$ (Nm/m)','Interpreter','latex','FontSize',11)
box on

subplot(2,1,2)
line([ 0.33 0.33],[0 8],'LineStyle','-.','Color',coal,'LineWidth',0.75)
hold on
line([ 0.84 0.84],[0 8],'LineStyle','-.','Color',coal,'LineWidth',0.75)
plot(t_tilde_f_plot,delta_seismic_max_plot*10^3,'o','MarkerSize',5,...
    'MarkerEdgeColor',orange,'MarkerFaceColor',orange)
hold off
xlabel('$\tilde t_f$ (-)','Interpreter','latex','FontSize',11)
ylabel('$\delta*_{max}$ (mm)','Interpreter','latex','FontSize',11)
box on


