function [data,flag,model,numpar] = ifs_patch_bounds(data,flag,model,numpar,plotpar)
%
% Computes the intersections y_pre(i), i = 1,...,4, of the combined shear stresses with the
% right slip threshold (i.e. the zeros of the pre-slip Coulomb stresses) and the boundaries
% y_til(i) of the slip patches (taking into account coupling between the patches) as a
% function of depletion pressure p_inc < 0. Uses a fixed or a variable pressure step size. 
%

% Retrieve:
% a = model.a;
b = model.b;
dir = numpar.dir; % indicates operation: dir = 1: injection; dir = -1: depletion, - 
dpdt = numpar.dpdt; % pressure drop per unit time, Pa/s
dp_dy_seis = numpar.dp_dy_seis; % threshold value of num. derivative to detect nucleation, Pa/m 
p_step_correct = numpar.p_step_correct; % correction factor for pressure step size, -
p_step_fix = numpar.p_step_fix; % fixed pressure step size (negative for depletion), Pa
p_step_init = numpar.p_step_init; % initial pressure step size (negative for depletion), Pa
n_p_max = numpar.n_p_max; % maximum number of pressure steps; for memory allocation, -
p_max_bounds = numpar.p_max_bounds; % highest incremental pressure (negative for depletion), Pa
p_min_bounds = numpar.p_min_bounds; % lowest incremental pressure (negative for depletion), Pa
y_step_max = numpar.y_step_max; % maximum allowed y_step value, m
small_patch = numpar.small_patch; % minimum patch size to trigger slip computation, m
tol_merge = numpar.tol_merge; % distance between slip patches to trigger merging, m  
if flag.plot_snap == 1
    i_snap_ref = plotpar.i_snap_ref; % snap shot serving as reference
    n_snap = plotpar.n_snap; % number of snap_shots, -
    p_inc_snap = plotpar.p_inc_snap; % incremental pressures at which to take snapshots, Pa
    n_pt = plotpar.y_plot.n_pt; % number of y data points, -
    snap = data.snap;
end

% Initialize:
flag.seismic = 0; % flag to indicate occurence of seismicity, -
C0 = zeros(1,n_p_max); % coefficient in coupled simulation, m^2 
C1 = zeros(1,n_p_max); % coefficient in coupled simulation, m
C0_sym = zeros(1,n_p_max); % coefficient in symmetric coupled simulation, m^2 
C0_short = zeros(1,n_p_max); % coefficient in coupled simulation, m^2 
C1_short = zeros(1,n_p_max); % coefficient in coupled simulation, m
C0_short_sym = zeros(1,n_p_max); % coefficient in symmetric coupled simulation, m^2 
Delta_y_eig_bot = zeros(1,n_p_max); % nucleation length bottom patch (eigenvalue-based), m
Delta_y_eig_top = zeros(1,n_p_max); % nucleation length top patch (eigenvalue-based), m
Delta_y_mer_bot = zeros(1,n_p_max); % bottom patch length at moment of merging, m
Delta_y_mer_top = zeros(1,n_p_max); % top patch length at moment of merging, m
Delta_y_sign_bot = zeros(1,n_p_max); % nucleation length bottom patch (sign-reversal-based), m
Delta_y_sign_mer = zeros(1,n_p_max); % nucleation length after merging (sign-reversal-based), m
Delta_y_sign_top = zeros(1,n_p_max); % nucleation length top patch (sign-reversal-based), m
Delta_y_UR_bot = zeros(1,n_p_max); % nucleation length bottom patch (Uenishi & Rice), m
Delta_y_UR_top = zeros(1,n_p_max); % nucleation length top patch (Uenishi & Rice), m
e_bot = zeros(1,n_p_max); % eigenvalue bottom patch, 1/m
e_top = zeros(1,n_p_max); % eigenvalue top patch, 1/m
j_seismic = 0; % pressure step at which seismicity occurs
N_cheb = numpar.N_cheb; % number of Gauss-Chebyshev integration points,-
E_bot = zeros(N_cheb/2,n_p_max); % eigenvector bottom patch, m/Pa
E_top = zeros(N_cheb/2,n_p_max); % eigenvector top patch, m/Pa
N_cheb_max = numpar.N_cheb_max; % maximum number of Gauss-Chebyshev integration points,-
p_inc = zeros(1,n_p_max); % incremental pressure, negative for depletion, Pa
p_seismic = 0; % depletion pressure at which seismicity occurs, Pa
p_step = zeros(1,n_p_max); % pressure step, Pa
y_step = zeros(1,n_p_max); % maximum observed change in Coulomb stress zeros or slip patch ...
    % boundaries, m 
epsilon_p = zeros(1,n_p_max); % -
rms_dR_dp = zeros(1,n_p_max); % root-mean-square value of dR_dp, - 
rms_dW_dp_delta = zeros(1,n_p_max); % root-mean-square value of dW_dp_delta, -
rms_W_ddelta_dp = zeros(1,n_p_max); % root-mean-square value of W_ddelta_dp, -
epsilon_py = zeros(1,n_p_max); % -
rms_dW_dy_ddelta_dp = zeros(1,n_p_max); % root-mean-square value of d2W_dpdy_delta, -
rms_W_d2delta_dpdy = zeros(1,n_p_max); % root-mean-square value of W_d2delta_dpdy, -
epsilon_y = zeros(1,n_p_max); % -
rms_dR_dy = zeros(1,n_p_max); % root-mean-square value of dR_dy, Pa/m 
rms_dW_dy_delta = zeros(1,n_p_max); % root-mean-square value of dW_dy_delta, Pa/m
rms_W_ddelta_dy = zeros(1,n_p_max); % root-mean-square value of W_ddelta_dy, Pa/m
y_pre = zeros(4,n_p_max);
y_til = zeros(4,n_p_max);
first_call_01 = 1;
if flag.plot_snap == 1
    plot_snap = zeros(n_snap,1); % flag to indicate that snap shot plot data have been ...
        % computed, -
    if flag.cond == 1 % correct conditions (default)
        y_pre_snap = zeros(4,n_snap);
        y_til_snap = zeros(4,n_snap);
        delta_snap = zeros(N_cheb_max,n_snap);
        grad_snap = zeros(N_cheb_max,n_snap);
        y_delta_snap = zeros(N_cheb_max,n_snap);
        sigma_brv_snap = zeros(n_pt,n_snap); % slip-induced stress, Pa        
        sigma_C_snap = zeros(n_pt,n_snap); % pre-slip Coulomb stress, Pa
        sigma_norm_eff_comb_snap = zeros(n_pt,n_snap); % combined effective normal stress, Pa
        sigma_shear_comb_snap = zeros(n_pt,n_snap); % combined shear stress, Pa
        sigma_slip_snap = zeros(n_pt,n_snap); % right slip threshold, Pa
        y_sigma_snap = zeros(n_pt,n_snap); % y values for stresses, m
    end
end
correct_step = 0; % flag to trigger correction of pressure step size, -
enough = 0; % flag to terminate pressure stepping, -
keep_step = 0; % flag to fix step size, -
merged_y_pre = 0; % flag to indicate merging of pre-slip Coulomb stress zeros y_pre(2) and ...
    % y_pre(3), -
merged_y_til = 0; % flag to indicate merging of slip patches y_til(2) and y_til(3), -
just_merged_y_pre = 0; % flag to indicate merging of pre-slip Coulomb stress zeros ...
    % y_pre(2) and y_pre(3) in current pressure step, -
just_merged_y_til = 0; % flag to indicate merging of slip patches y_til(2) and y_til(3) in ...
    % current pressure step, -    

% Start simulation:
if flag.echo > 0
    if flag.fric == 1
        fprintf('\rSimulating fault slip with constant friction.\r');
    end
    if flag.fric == 2
        fprintf('\rSimulating fault slip with slip-weakening friction.\r');
    end
end
j = 0; % pressure step counter
while enough == 0
    j = j+1;
    if merged_y_til == 0
        j_y_til = j-1; % dummy
    end
    if j > n_p_max
        error('Maximum number of pressure steps numpar.n_p_max has been exceeded.');
    end
    
    % Determine incremental pressure:
    if j == 1
        p_inc(j) = dir*min(dir*p_min_bounds,dir*p_max_bounds); % initial incremental ...
            % pressure, negative for depletion, Pa
    end
    flag_print_correct_step = 0; % local flag
    if flag.varstep == 1
        if j == 1 || j == 2 || keep_step == 1
            p_step(j) = p_step_init;
        else           
            switch correct_step
                case {0,3} % computes new pressure step
                    p_step_new = ifs_pressure_step_predict(numpar,j,p_step,y_pre,y_til);
                    correct_step = 0; % resets flag after correction in previous step
                case 1 % repeats step with smaller value
                    p_step_new = p_step_correct * p_step(j); % 0 < p_step_correct < 1
                    flag_print_correct_step = 1;
                    correct_step = 3; % avoids another correction in current step
                case 2 % sets step just after merging
                    p_step_new = p_step(j-1); 
                    correct_step = 3; % avoids another correction in current step
            end
                p_step(j) = ifs_pressure_step_check(numpar,p_step_new,p_step(j-1)); % checks ...
                    % absolute and relative growth or shrinkage and corrects if necessary
        end
    else
        p_step(j) = p_step_fix; % fixed step size
    end
    if j > 1
        p_inc(j) = p_inc(j-1) + p_step(j);
    end
    
    % Check if pressure step triggers a snap shot plot, and if so, temporarily reset p_step ...
        % and p_inc, and compute snap shot data:
    i_snap_act = 0;
    if flag.plot_snap == 1 && flag.varstep == 1  
        for i_snap = 1:n_snap 
            if p_inc(j) <= p_inc_snap(i_snap) && i_snap_act == 0 && plot_snap(i_snap) == 0
                i_snap_act = i_snap; 
                p_inc(j) = p_inc_snap(i_snap); % resets p_inc to predefined snap shot pressure
                p_step(j) = p_inc(j) - p_inc(j-1); % computes corresponding step size p_step
                plot_snap(i_snap) = 1; 
            end
        end
    end
    
    % Check if final pressure has been reached:
    if dir == 1 % injection
        if p_inc(j) > p_max_bounds
            p_inc(j) = p_max_bounds;
        end
    else % depletion
        if p_inc(j) < p_min_bounds % p_inc and p_min_bounds are negative for depletion
            p_inc(j) = p_min_bounds;
        end
    end
    
    % Compute pre-slip Coulomb stress zeros y_pre: 
    fric_save = 0;
    if j == 1 || keep_step == 1
        fric_save = flag.fric;
        flag.fric = 1; % local value
    end
    y_pre(:,j) = ifs_y_pre(flag,model,numpar,p_inc(j));
    if fric_save ~= 0
        flag.fric = fric_save; % resets to original value
    end
    if y_pre(3,j) > y_pre(2,j) % no merging yet
        j_y_pre = j; % last pressure step at which no merging has occurred 
    else
        y_pre_av = (y_pre(2,j-1) + y_pre(3,j-1)) / 2; % average value just before merging, m
        y_pre(2,j) = y_pre_av; % taken as value after merging, m
        y_pre(3,j) = y_pre_av;
        if merged_y_pre == 0
            just_merged_y_pre = 1; % flag to indicate recent merging
            merged_y_pre = 1; % flag to indicate merging
        end
    end

    % Compute total number of Gauss-Chebyshev integration points N_cheb (distributed over ...
    % patches) and number of points N_bot, N_top and N_mer per patch: 
    if j == 1
        numpar = ifs_compute_points(flag,numpar,y_pre(:,j));
    else
        numpar = ifs_compute_points(flag,numpar,y_til(:,j-1));
    end

    % Write to screen:
    N_cheb = numpar.N_cheb;
    if flag.echo == 2
        if flag_print_correct_step == 1
            fprintf('\rPressure step %-3.0i (repeat)              p_inc = %7.4f MPa, N_cheb = %3.0i\r',j,...
                p_inc(j)/1e6,N_cheb);
        else
            fprintf('\rPressure step %-3.0i                       p_inc = %7.4f MPa, N_cheb = %3.0i\r',j,...
                p_inc(j)/1e6,N_cheb);
        end
    end

    % Compute slip patch boundaries y_til; see ifs_auxiliary.m for the initial setting of ...
        % flag.sim_type wich determines if a single-patch, or double patch solution is ...
        % computed in ifs_y_til.m.:
    if abs(y_pre(2,j)-y_pre(1,j)) < small_patch || abs(y_pre(4,j)-y_pre(3,j)) < small_patch
        keep_step = 1; % keeps pressure increment the same for the next step
        y_til(:,j) = y_pre(:,j); % skips computation of y_til
    else % both patches larger than very small
        % Determine first guess:
        fric_save = 0;
        if j == 1 || keep_step == 1
            y_til_0 = y_pre(:,j); % first guess for slip patch bounds
            fric_save = flag.fric; % local value
            flag.fric = 1; % forces use of constant friction during initial steps
            keep_step = 0; % recompute pressure increment at next step
        else
            if abs(y_til(2,j-1)-y_til(1,j-1)) < small_patch || ...
                    abs(y_til(4,j-1)-y_til(3,j-1)) < small_patch
                y_til_0 = y_pre(:,j); % first guess for slip patch bounds
            else
                y_til_0 = y_til(:,j-1); % takes result from previous pressure step as f. guess
            end
        end
        % Compute slip patch boundaries, slip gradient and slip:
        [numpar,delta,grad,y_delta,y_til(:,j)] = ifs_y_til(flag,model,numpar,plotpar,...
            p_inc(j),y_til_0);
        if fric_save ~= 0
            flag.fric = fric_save; % resets to original value
        end
        % Compute coefficients C0 and C1 for coupled simulation (for diagnostics only):
        if flag.sim_type == 2 && flag.plot_diagn_C == 1 && flag.fric == 1 && flag.int ~=3
            if flag.sym == 1 % indicates symmetry around y = 0
                [C0(j),C0_short(j)] = ifs_compute_CD_sym(flag,model,numpar,p_inc(j),...
                    y_til(3:4,j));
                if flag.plot_asy == 1
                    C0_sym(j) = C0(j);
                    C0_short_sym(j) = C0_short(j);
                end
            else % asymmetric configuration
                [C0(j),C0_short(j),C1(j),C1_short(j)] = ifs_compute_CD(flag,model,numpar,...
                    delta,p_inc(j),y_delta,y_til(:,j));
            end
        end
        % Compute root-mean-squared values of loading stress components (for diagnostics only):
        if flag.plot_diagn == 1 && flag.fric == 2 && flag.sim_type ~= 3 
            % Compute dR_dy, dW_dy_delta, W_ddelta_dy and epsilon_y for the top patch:
            N_delta = length(y_delta);
            delta_top = delta(N_delta/2+1:end-1);
            grad_top = grad(N_delta/2+1:end-1);
            y_delta_top = y_delta(N_delta/2+1:end-1);
            [epsilon_y(j),rms_dR_dy(j),rms_dW_dy_delta(j),rms_W_ddelta_dy(j)] = ...
            ifs_loading_stress_components_dy(flag,model,numpar,delta_top,grad_top,p_inc(j),...
            y_delta_top);
            if first_call_01 == 1
                delta_top_store = delta_top; % initialization
                grad_top_store = delta_top; % initialization
                y_delta_top_store = y_delta_top; % same
                first_call_01 = 0;
            end
            if j > 2
                % Compute dR_dp, dW_dp_delta, W_ddelta_dp and epsilon_p for the top patch:
                [epsilon_p(j),rms_dR_dp(j),rms_dW_dp_delta(j),rms_W_ddelta_dp(j)] = ...
                    ifs_loading_stress_components_dp(flag,model,numpar,delta_top,...
                    delta_top_store,p_inc(j),p_inc(j-1),y_delta_top,y_delta_top_store);
                % Compute dW_dy_ddelta_dp, W_d2delta_dpdy and epsilon_py for the top patch:
                [epsilon_py(j),rms_dW_dy_ddelta_dp(j),rms_W_d2delta_dpdy(j)] = ...
                    ifs_loading_stress_components_dpdy(flag,model,numpar,delta_top,...
                    delta_top_store,grad_top,grad_top_store,p_inc(j),p_inc(j-1),y_delta_top,...
                    y_delta_top_store);
            end
            delta_top_store = delta_top; % to compute numerical derivative in next pr. step
            y_delta_top_store = y_delta_top; % same
        end

        % Check for seismicity (before merging):
        if flag.sim_type ~= 3 && flag.fric == 2 && j > 1 && keep_step == 0 && flag.seismic == 0
            if flag.int == 3
                % Compute eigen values and eigen vectors:
                [e_bot(j),e_top(j),E_bot(:,j),E_top(:,j)] = ifs_eigen(flag,model,numpar,...
                    delta,p_inc(j),y_delta,y_til(:,j));
            end
            % Compute (Extended) Uenishi & Rice citerion:
            [Delta_y_UR_bot(j),Delta_y_UR_top(j)] = ifs_UR(flag,model,numpar,delta,...
                p_inc(j),y_delta,y_til(:,j));
            switch flag.nuc % flag to select detection method for nucleation
                case 1 % Use a numerical derivative (inverse grad. dp/dy_til in Pa/m):
                    y_step_til_obs_bot = max(abs(y_til(1:2,j) - y_til(1:2,j-1))); % maximum ...
                    % observed change in bottom slip patch boundaries, -
                    dp_dy_obs_bot = abs( p_step(j) / y_step_til_obs_bot );
                    if dp_dy_obs_bot < dp_dy_seis
                        flag.seismic = 1;
                        j_seismic = j; % pressure step at which seismicity occurs
                        p_seismic = p_inc(j); % depletion pressure at which seismicity occurs
                        Delta_y_obs_bot_prev = y_til(2,j-1)-y_til(1,j-1); 
                        if flag.echo > 0
                            fprintf('*** Seismic event has occured in bottom patch (derivative criterion) ***\r');
                            fprintf('p* = %g MPa\r',p_seismic/1e6);
                            fprintf('Delta_y*_deriv = %g m\r',Delta_y_obs_bot_prev);
                            fprintf('Delta_y*_UR = %g m\r',Delta_y_UR_bot(j-1));
                        end
                    end
                    y_step_til_obs_top = max(abs(y_til(3:4,j) - y_til(3:4,j-1))); % maximum ...
                    % observed change in top slip patch boundaries, -
                    dp_dy_obs_top = abs( p_step(j) / y_step_til_obs_top );
                    if dp_dy_obs_top < dp_dy_seis
                        flag.seismic = 1;
                        j_seismic = j; % pressure step at which seismicity occurs
                        p_seismic = p_inc(j); % depletion pressure at which seismicity occurs
                        Delta_y_obs_top_prev = y_til(4,j-1)-y_til(3,j-1); 
                        if flag.echo > 0
                            fprintf('*** Seismic event has occured in top patch (derivative criterion) ***\r');
                            fprintf('p* = %g MPa\r',p_seismic/1e6);
                            fprintf('Delta_y*_deriv = %g m\r',Delta_y_obs_top_prev);
                            fprintf('Delta_y*_UR = %g m\r',Delta_y_UR_top(j-1));
                        end
                    end
                case 2 % Use an eigenvalue analysis:
                    Delta_y_bot = y_til(2,j)-y_til(1,j); % bottom patch length, m
                    Delta_y_eig_bot(j) = Delta_y_bot/e_bot(j); % nucleation length bottom patch, m
                    if Delta_y_bot > Delta_y_eig_bot(j) && flag.sym == 0
                        flag.seismic = 1;
                        j_seismic = j; % pressure step at which seismicity occurs
                        p_seismic = p_inc(j); % depletion pressure at which seismicity occurs
                        if flag.echo > 0
                            fprintf('*** Seismic event has occured in bottom patch (eigenvalue criterion) ***\r');
                            fprintf('p* = %g MPa\r',p_seismic/1e6);
                            fprintf('Delta_y*_eig = %g m\r',Delta_y_eig_bot(j));
                            fprintf('Delta_y*_UR = %g m\r',Delta_y_UR_bot(j));
                        end
                    end
                    Delta_y_top = y_til(4,j)-y_til(3,j);
                    Delta_y_eig_top(j) = Delta_y_top/e_top(j);
                    if Delta_y_top > Delta_y_eig_top(j) 
                        flag.seismic = 1;
                        j_seismic = j; % pressure step at which seismicity occurs
                        p_seismic = p_inc(j); % depletion pressure at which seismicity occurs
                        if flag.echo > 0
                            fprintf('*** Seismic event has occured in top patch (eigenvalue citerion) ***\r');
                            fprintf('p* = %g MPa\r',p_seismic/1e6);
                            fprintf('Delta_y*_eig = %g m\r',Delta_y_eig_top(j));
                            fprintf('Delta_y*_UR = %g m\r',Delta_y_UR_top(j));
                        end
                    end
                case 3 % Use the Uenishi and Rice criterion:
                    Delta_y_bot = y_til(2,j)-y_til(1,j); % bottom patch length, m
                    if Delta_y_bot > Delta_y_UR_bot(j-1) && Delta_y_UR_bot(j-1) > 0 && ...
                            flag.sym == 0 
                        flag.seismic = 1;
                        j_seismic = j; % pressure step at which seismicity occurs
                        p_seismic = p_inc(j); % depletion pressure at which seismicity occurs
                        if flag.echo > 0
                            fprintf('*** Seismic event has occured in bottom patch (U&R criterion) ***\r');
                            fprintf('p* = %g MPa\r',p_seismic/1e6);
                            fprintf('Delta_y*_UR = %g m\r',Delta_y_UR_bot(j));
                        end
                    end
                    Delta_y_top = y_til(4,j)-y_til(3,j);
                    if Delta_y_top > Delta_y_UR_top(j) && Delta_y_UR_top(j) > 0
                        flag.seismic = 1;
                        j_seismic = j; % pressure step at which seismicity occurs
                        p_seismic = p_inc(j); % depletion pressure at which seismicity occurs
                        if flag.echo > 0
                            fprintf('*** Seismic event has occured in top patch (U&R criterion) ***\r');
                            fprintf('p* = %g MPa\r',p_seismic/1e6);
                            fprintf('Delta_y*_UR = %g m\r',Delta_y_UR_top(j-1));
                        end
                    end
                case 4 % Use the patch-growth sign reversal criterion:
                    % Check for spurious reduction in slip patch length:
                    Delta_y_top_prev = y_til(4,j-1)-y_til(3,j-1);
                    Delta_y_top = y_til(4,j)-y_til(3,j);
                    Delta_y_eig_top(j-1) = Delta_y_top_prev/e_top(j-1); 
                    Delta_y_sign_top(j-1) = Delta_y_top_prev; 
                    if Delta_y_top < Delta_y_top_prev
                        flag.seismic = 1;
                        j_seismic = j-1; % pressure step at which seismicity occurs
                        p_seismic = p_inc(j-1); % depletion pressure at which seismicity occurs
                        if flag.echo > 0
                            fprintf('*** Seismic event has occured in top patch (sign reversal criterion) ***\r');
                            fprintf('p* = %g MPa\r',p_seismic/1e6);
                            fprintf('Delta_y*_sign = %g m\r',Delta_y_sign_top(j-1));
                            fprintf('Delta_y*_eig = %g m\r',Delta_y_eig_top(j-1));
                            fprintf('Delta_y*_UR = %g m\r',Delta_y_UR_top(j-1));
                        end
                    end
                    Delta_y_bot_prev = y_til(2,j-1)-y_til(1,j-1);
                    Delta_y_bot = y_til(2,j)-y_til(1,j);
                    Delta_y_eig_bot(j-1) = Delta_y_bot_prev/e_bot(j-1); 
                    Delta_y_sign_bot(j-1) = Delta_y_bot_prev; 
                    if Delta_y_bot < Delta_y_bot_prev && flag.seismic == 0
                        flag.seismic = 1;
                        j_seismic = j-1; % pressure step at which seismicity occurs
                        p_seismic = p_inc(j-1); % depletion pressure at which seismicity occurs
                        if flag.echo > 0
                            fprintf('*** Seismic event has occured in bottom patch (sign reversal criterion) ***\r');
                            fprintf('p* = %g MPa\r',p_seismic/1e6);
                            fprintf('Delta_y*_sign = %g m\r',Delta_y_sign_bot(j-1));
                            fprintf('Delta_y*_eig = %g m\r',Delta_y_eig_bot(j-1));
                            fprintf('Delta_y*_UR = %g m\r',Delta_y_UR_bot(j-1));
                        end
                    end
            end
        end

        % Check for merging:
        if flag.seismic == 0
            if flag.sym == 0
                if y_til(3,j) - y_til(2,j) > tol_merge % no merging yet
                    j_y_til = j; % last pressure step at which no merging has occurred
                else
                    y_til_av = (y_til(2,j-1) + y_til(3,j-1)) / 2; % average value just ...
                        % before merging, m
                    y_til(2,j) = y_til_av; % taken as value after merging, m
                    y_til(3,j) = y_til_av;
                    p_mer = p_inc(j); % depletion pressure at which merging occurs
                    Delta_y_mer_bot(j) = y_til_av - y_til(1,j); 
                    Delta_y_mer_top(j) = y_til(4,j) - y_til_av; 
                    if merged_y_til == 0
                        just_merged_y_til = 1; % flag to indicate recent merging
                        merged_y_til = 1; % flag to indicate merging
                        fprintf('*** Slip patches have merged ***\r');
                        fprintf('p_mer = %g MPa\r',p_mer/1e6);
                        flag.sim_type = 3; % merged
                    end
                end
            else
                if y_til(3,j) > tol_merge % no merging yet
                    j_y_til = j; % last pressure step at which no merging has occurred
                    fprintf('y_til_3 = %g m\r',y_til(3,j));
                else
                    fprintf('y_til_3 = %g m\r',y_til(3,j));
                    y_til_3_last = y_til(3,j);
                    y_til(2,j) = 0; % taken as value after merging, m
                    y_til(3,j) = 0;
                    if merged_y_til == 0
                        p_mer = p_inc(j); % depletion pressure at which merging occurs
                        Delta_y_mer_bot(j) = -y_til(1,j); 
                        Delta_y_mer_top(j) =  y_til(4,j); 
                        just_merged_y_til = 1; % flag to indicate recent merging
                        merged_y_til = 1; % flag to indicate merging
                        fprintf('*** Slip patches have merged ***\r');
                        fprintf('p_mer = %g MPa\r',p_mer/1e6);
                        fprintf('y_til_3 = %g m\r',y_til_3_last);
                        flag.sim_type = 3; % merged
                    end
                end
            end
        else
            j_y_til = j-1; % dummy
        end
    end
   
    % Compute snap shot data:
    if i_snap_act ~= 0
        switch flag.cond % flag to set coupling conditions
            case 1 % correct conditions (default)
                y_pre_snap(:,i_snap_act) = y_pre(:,j); % pre-slip Coulomb stress zeros, m
                y_til_snap(:,i_snap_act) = y_til(:,j); % slip patch boundaries (coupled), m
                N_delta = length(y_delta);
                delta_snap(1:N_delta,i_snap_act) = delta;
                grad_snap(1:N_delta,i_snap_act) = grad;
                y_delta_snap(1:N_delta,i_snap_act) = y_delta;
                [numpar,sigma_snap,sigma_brv_snap(:,i_snap_act),...
                    y_sigma_snap(:,i_snap_act)] = ifs_compute_snaps(flag,model,numpar,...
                    grad,p_inc(j),plotpar,y_delta,y_til(:,j));
                sigma_C_snap(:,i_snap_act) = sigma_snap.C';
                sigma_norm_eff_comb_snap(:,i_snap_act) = sigma_snap.norm_eff_comb;
                sigma_shear_comb_snap(:,i_snap_act) = sigma_snap.shear_comb';
                sigma_slip_snap(:,i_snap_act) = sigma_snap.slip';
            case 2 % incorrect Mushkelishvili conditions
                if i_snap_act == i_snap_ref % reference snap shot is active 
                    y_til_mus = y_til(:,j); 
                    delta_snap_mus = delta;
                    grad_snap_mus = grad;
                    y_delta_snap_mus = y_delta;
                end
            case 3 % incorrect Weertman conditions
                if i_snap_act == i_snap_ref % reference snap shot is active 
                    y_til_wee = y_til(:,j); 
                    delta_snap_wee = delta;
                    grad_snap_wee = grad;
                    y_delta_snap_wee = y_delta;
                end
        end
    end

    % Check y_step size and set trigger for correction of pressure step size if necessary; ...
    % also reset the temporary values of p_inc and p_step for snapshots; only one correction
    % per pressure step is allowed: 
    if flag.varstep == 1 && 2 < j && correct_step ~= 3
        [correct_step,y_step(j)] = ifs_pressure_step_correct(numpar,j,y_pre,y_til);
        if correct_step == 1 || i_snap_act ~= 0 % triggers correction
           if just_merged_y_pre == 0 && just_merged_y_til == 0
                j = j-1; % pressure step will be repeated
                keep_step = 0;
           else % avoids step size correction just after merging
                correct_step = 2; % triggers use of small step just after merging
                just_merged_y_pre = 0;
                just_merged_y_til = 0;
                y_step(j) = y_step_max; % cosmetic
           end
        end
    end
    
    % Check for completion:
    if dir == 1 % injection
        if p_inc(j) < p_max_bounds && flag.seismic == 0
            enough = 0;
        else
            enough = 1; % will terminate loop
        end
    else % depletion
        if p_inc(j) > p_min_bounds && ... % note: p_inc < 0 for depletion
                (flag.seismic == 0 || flag.seismic == 1 && correct_step == 1) && ...
                (merged_y_til == 0 || flag.fric == 1) % stops after merging in case of ...
            % slip-weakening friction!
            enough = 0;
        else
            enough = 1;
            if flag.nuc == 4 && flag.seismic == 1  
                j = j-1; % because all results have been taken from step j-1 
            end
        end
    end
end

% Compute aseismic moment per unit strike-length m*:
if merged_y_til == 1
    delta_mer = delta(2:end-1);
    y_delta_mer = y_delta(2:end-1);
    dy_delta_mer = ifs_cdiff(y_delta_mer);
    m_star = model.G * sum(delta_mer.*dy_delta_mer); 
else
    N = N_cheb/2;
    delta_bot = delta(2:N+1);
    delta_top = delta(N+2:end-1);
    y_delta_bot = y_delta(2:N+1);
    y_delta_top = y_delta(N+2:end-1);
    dy_delta_bot = ifs_cdiff(y_delta_bot);
    dy_delta_top = ifs_cdiff(y_delta_top);
    m_star = model.G * ( sum(delta_bot.*dy_delta_bot) + sum(delta_top.*dy_delta_top) ); 
end

% Collect output data:
if flag.plot_snap == 1
    switch flag.cond % flag to set coupling conditions
        case 1 % correct conditions (default)
            snap.y_pre = y_pre_snap; % pre-slip Coulomb stress zeros, m
            snap.y_til = y_til_snap; % slip patch boundaries (coupled), m
            snap.delta = delta_snap; % slip, m
            snap.grad = grad_snap; % slip gradient, -
            snap.sigma_brv = sigma_brv_snap; % slip-induced stress, Pa
            snap.sigma_C = sigma_C_snap; % pre-slip Coulomb stress, Pa
            snap.sigma_norm_eff_comb = sigma_norm_eff_comb_snap; % comb. eff. normal str., Pa 
            snap.sigma_shear_comb = sigma_shear_comb_snap; % combined shear stress, Pa
            snap.sigma_slip = sigma_slip_snap; % right slip threshold, Pa
            snap.y_delta = y_delta_snap; % y values for slip and slip gradient, m
            snap.y_sigma = y_sigma_snap; % y values for stresses, m
        case 2 % incorrect Mushkelishvili conditions
            snap.delta_mus = delta_snap_mus;
            snap.grad_mus = grad_snap_mus;
            snap.y_delta_mus = y_delta_snap_mus;
            snap.y_til_mus = y_til_mus;
        case 3 % incorrect Weertman conditions
            snap.delta_wee = delta_snap_wee;
            snap.grad_wee = grad_snap_wee;
            snap.y_delta_wee = y_delta_snap_wee;
            snap.y_til_wee = y_til_wee;
    end
    data.snap = snap; % various snap shot data
end
data.j_merge_y_pre = j_y_pre + 1; % pressure step at which pre-slip Coulomb stress patches ...
    % have merged, -
data.j_merge_y_til = j_y_til + 1; % pressure step at which slip patches have merged, -
data.j_seismic = j_seismic; % pressure step at which seismicity occurs, -
data.coeff_C0 = C0(1:j); % coefficient in coupled simulation, m^2 
data.coeff_C1 = C1(1:j); % coefficient in coupled simulation, m 
data.coeff_C0_short = C0_short(1:j);  
data.coeff_C1_short = C1_short(1:j); 
if flag.sym == 1 
    data.coeff_C0_sym = C0_sym(1:j); % coefficient in symmetric coupled simulation, m^2
    data.coeff_C0_short_sym = C0_short_sym(1:j);
end
data.Delta_y_eig_bot = Delta_y_eig_bot(1:j); % nucl. length bottom patch (eigenvalue-based), m
data.Delta_y_eig_top = Delta_y_eig_top(1:j); % nucl. length top patch (eigenvalue-based), m
data.Delta_y_mer_bot = Delta_y_mer_bot(1:j); % bottom patch length at moment of merging, m
data.Delta_y_mer_top = Delta_y_mer_top(1:j); % top patch length at moment of merging, m
data.Delta_y_sign_bot = Delta_y_sign_bot(1:j); % nucleation length bottom patch (sign-reversal), m
data.Delta_y_sign_top = Delta_y_sign_top(1:j); % nucleation length top patch (sign-reversal), m
data.Delta_y_UR_bot = Delta_y_UR_bot(1:j); % nucleation length bottom patch (Uenishi & Rice), m
data.Delta_y_UR_top = Delta_y_UR_top(1:j); % nucleation length top patch (Uenishi & Rice), m
data.e_bot = e_bot(1:j); % eigenvalue bottom patch, 1/m
data.e_top = e_top(1:j); % eigenvalue top patch, 1/m
data.E_bot = E_bot(:,1:j); % eigenvector bottom patch, m/Pa
data.E_top = E_top(:,1:j); % eigenvector top patch, m/Pa
data.m_star = m_star; % aseismic moment per unit strike length
data.p_inc = p_inc(1:j); % incremental pressure; negative for depletion, Pa
data.p_seismic = p_seismic; % depletion pressure at which seismicity occurs, Pa
data.p_step = p_step(1:j); % pressure step history, Pa
data.y_pre = y_pre(:,1:j); % pre-slip Coulomb stress zeros, m
data.y_til = y_til(:,1:j); % slip patch boundaries (coupled), m
data.y_step = y_step(1:j); % maximum slip patch boundary growth history, m 
data.epsilon_p = epsilon_p(1:j); % -
data.rms_dR_dp = rms_dR_dp(1:j); % root-mean-square value of dR_dp, - 
data.rms_dW_dp_delta = rms_dW_dp_delta(1:j); % root-mean-square value of dW_dp_delta, -
data.rms_W_ddelta_dp = rms_W_ddelta_dp(1:j); % root-mean-square value of W_ddelta_dp, -
data.epsilon_y = epsilon_y(1:j); % -
data.rms_dR_dy = rms_dR_dy(1:j); % root-mean-square value of dR_dy, Pa/m 
data.rms_dW_dy_delta = rms_dW_dy_delta(1:j); % root-mean-square value of dW_dy_delta, Pa/m
data.rms_W_ddelta_dy = rms_W_ddelta_dy(1:j); % root-mean-square value of W_ddelta_dy, Pa/m
data.epsilon_py = epsilon_py(1:j); % -
data.rms_dW_dy_ddelta_dp = rms_dW_dy_ddelta_dp(1:j); % rms value of d2W_dpdy_delta, 1/m
data.rms_W_d2delta_dpdy = rms_W_d2delta_dpdy(1:j); % rms value of W_d2delta_dpdy, 1/m
data.delta_seismic = delta; % slip at nucleation, m
data.y_delta_seismic = y_delta; % y values of delta at nucleation, m


