function ifs_plot(admin,data,flag,model,numpar,plotpar)
%
% Generates various plots.
%
% The first three plot files (ifs_plot_images.m, ifs_plot_const_x.m and ifs_plot_fido.m) make
% use of inclusion theory to illustrate pre-slip features (displacements, stresses) in the
% simulation domain.     
%
% The file ifs_plot_patch_bounds.m generates the main plot: slip patch boundaries as a function
% of incremental pressure. 
%

% Create image plots of pre-slip displacements (for vertical faults only) and stresses.
if flag.plot_image == 1
    ifs_plot_images(flag,model,plotpar);
end

% Create plots of pre-slip stresses for constant x values:
if flag.plot_const_x == 1
    ifs_plot_const_x(flag,model,plotpar);
end

% Compute and plot pre-slip stresses and displacements at the boundaries of a finite domain.
if flag.fido == 1
    % Compute scaled stresses and displacements:
    [epsilon_tilde,sigma_tilde,u_tilde] = ifs_fido(flag,model,plotpar);

    % Plot stresses and displacements:
    ifs_plot_fido(flag,model,plotpar,epsilon_tilde,sigma_tilde,u_tilde);
end

% Plot & print simulation results:
if flag.fric == 0
    % Plot stresses and displacements in a frictionless fault for p_ref:
    ifs_plot_frictionless(data,flag,plotpar)
else
    % Plot pre-slip Coulomb stress zeros and slip patch boundaries as function of depletion:
    ifs_plot_patch_bounds(admin,data,flag,model,numpar,plotpar);

    % Plot nucleation lengths as function of depletion: 
    if flag.fric == 2 && flag.sim_type ~= 3 % slip weakening; not merged
        ifs_plot_nucleation(data,flag,model,plotpar);
    end

    % Plot snapshots:
    if flag.plot_snap == 1
        ifs_plot_snaps(admin,data,flag,model,plotpar);
    end

    % Plot pressure stepping history (for diagnostics):
    if flag.plot_diagn == 1 && flag.varstep == 1
        ifs_plot_step_history(data,flag,numpar,plotpar);
    end

    % Plot (derivative of) pre-slip Coul. str. close to singularity at y = a (for diagnostics):
    if flag.plot_diagn == 1 && flag.plot_snap == 1 && flag.fric == 1 && flag.inf == 1 && ...
            admin.input ~= 03 % excludes Buijze et al. (2019) example
        ifs_plot_d_sigma_C_dy(data,flag,model,plotpar);
    end

    % Plot coefficients C0 and C1 for coupled simulation (for diagnostics):
    if flag.plot_diagn_C == 1 && flag.fric == 1 && flag.int ~=3 
        ifs_plot_C(admin,data,flag,numpar,plotpar);
    end

    % Plot root-mean-squared values of the loading stress components dR_dp, dW_dp_delta and ...
    % W_ddelta_dp, and epsilon_p for the top patch (for diagnostics only):
    if flag.plot_diagn == 1 && flag.fric == 2 && flag.sim_type ~= 3
        ifs_plot_loading_stress_components_dp(data,flag,plotpar)
    end

    % Plot root-mean-squared values of the loading stress components d2W_dpdy_delta and ...
    % W_d2delta_dpdy, and epsilon_py for the top patch (for diagnostics only):
    if flag.plot_diagn == 1 && flag.fric == 2 && flag.sim_type ~= 3
        ifs_plot_loading_stress_components_dpdy(data,flag,plotpar)
    end

    % Plot root-mean-squared values of the loading stress components dR_dy, dW_dy_delta and ...
    % W_ddelta_dy, and epsilon_y for the top patch (for diagnostics only):
    if flag.plot_diagn == 1 && flag.fric == 2 && flag.sim_type ~= 3
        ifs_plot_loading_stress_components_dy(data,flag,plotpar)
    end
end




