function ifs_plot_fido(flag,model,plotpar,epsilon_tilde,sigma_tilde,u_tilde)
%
% Creates plots of pre-slip stresses and displacements for reference pressure p_inc_ref at
% specified constant values of x and y that represent the boundaries of a finite domain. 
%

% Retrieve parameters:
H = model.H; % height of finite domain, m
W = model.W; % width of finite domaion, m

n_fido = plotpar.n_fido; % nr. of points to plot strains and stresses at domain boundaries, -
p_inc_ref = plotpar.p_inc_ref; % reference pressure used in image plots etc., Pa

black = plotpar.colors.black;
blue = plotpar.colors.blue;
green = plotpar.colors.green;
red = plotpar.colors.red;

% Compute auxiliary variables:
x_E =  W/2; % x value @ East boundary
x_W = -W/2; % x value @ West boundary
y_N =  H/2; % y value @ North boundary
y_S = -H/2; % y value @ South boundary
x_NS = linspace(x_W,x_E,n_fido); % x values @ North and South boundaries 
y_EW = linspace(y_S,y_N,n_fido); % y values @ East and West boundaries

sigma_xx_E = p_inc_ref * sigma_tilde.xx_E; 
sigma_xx_W = p_inc_ref * sigma_tilde.xx_W; 
sigma_xx_N = p_inc_ref * sigma_tilde.xx_N; 
sigma_xx_S = p_inc_ref * sigma_tilde.xx_S; 
sigma_yy_E = p_inc_ref * sigma_tilde.yy_E; 
sigma_yy_W = p_inc_ref * sigma_tilde.yy_W; 
sigma_yy_N = p_inc_ref * sigma_tilde.yy_N; 
sigma_yy_S = p_inc_ref * sigma_tilde.yy_S; 
sigma_xy_E = p_inc_ref * sigma_tilde.xy_E; 
sigma_xy_W = p_inc_ref * sigma_tilde.xy_W; 
sigma_xy_N = p_inc_ref * sigma_tilde.xy_N; 
sigma_xy_S = p_inc_ref * sigma_tilde.xy_S; 

if flag.vert == 1 % vertical fault
    u_x_E = p_inc_ref * u_tilde.x_E;
    u_x_W = p_inc_ref * u_tilde.x_W;
    u_x_N = p_inc_ref * u_tilde.x_N;
    u_x_S = p_inc_ref * u_tilde.x_S;
    u_y_E = p_inc_ref * u_tilde.y_E;
    u_y_W = p_inc_ref * u_tilde.y_W;
    u_y_N = p_inc_ref * u_tilde.y_N;
    u_y_S = p_inc_ref * u_tilde.y_S;
    % Check: recompute vertical displacements at E boundary from strains:
    epsilon_yy_E = p_inc_ref * epsilon_tilde.yy_E;
    Delta_y_EW = H/n_fido;
    u_y_E_check = u_y_E(1) + cumtrapz(epsilon_yy_E * Delta_y_EW);
    epsilon_yy_W = p_inc_ref * epsilon_tilde.yy_W;
    u_y_W_check = u_y_W(1) + cumtrapz(epsilon_yy_W * Delta_y_EW);
    % Check: recompute horizontal displacements at N boundary from strains:
    epsilon_xx_N = p_inc_ref * epsilon_tilde.xx_N;
    Delta_x_NS = W/n_fido;
    u_x_N_check = u_x_N(1) + cumtrapz(epsilon_xx_N * Delta_x_NS);
    epsilon_xx_S = p_inc_ref * epsilon_tilde.xx_S;
    u_x_S_check = u_x_S(1) + cumtrapz(epsilon_xx_S * Delta_x_NS);
end

% Create plots:
% Stresses at E and W boundaries:
figure
y_min = min(y_EW);
y_max = max(y_EW);
subplot(1,3,1)
x_min = min(sigma_xx_E/1e6);
x_max = max(sigma_xx_E/1e6);
hold on
plot(sigma_xx_E/1e6,y_EW,'-','Color',blue,'LineWidth',1.0)
plot(sigma_xx_W/1e6,y_EW,'--','Color',red,'LineWidth',1.0)
line([x_min x_max ],[0 0],'LineStyle','-','Color',black,'LineWidth',0.5)
line([0 0],[y_min y_max ],'LineStyle','-','Color',black,'LineWidth',0.5)
hold off
axis([x_min x_max y_min y_max])
xlabel('$\it \sigma_{xx} \; \rm (MPa)$','Interpreter','latex','FontSize',11)
ylabel('$\it y \; \rm (m)$','Interpreter','latex','FontSize',11)
box on
legend('$E$','$W$','Interpreter','latex','FontSize',11,'location','SouthEast')

subplot(1,3,2)
x_min = min(sigma_yy_E/1e6);
x_max = max(sigma_yy_E/1e6);
hold on
line([x_min x_max ],[0 0],'LineStyle','-','Color',black,'LineWidth',0.5)
line([0 0],[y_min y_max ],'LineStyle','-','Color',black,'LineWidth',0.5)
plot(sigma_yy_E/1e6,y_EW,'-','Color',blue,'LineWidth',1.0)
plot(sigma_yy_W/1e6,y_EW,'--','Color',red,'LineWidth',1.0)
hold off
axis([x_min x_max y_min y_max])
if flag.plot_cap == 1
    x_text = x_min + 0.25 * (x_max-x_min);  
    y_text = y_max + 0.05 * (y_max-y_min);
    text(x_text,y_text,'plot\_fido','FontSize',10,'Color',red)
end
xlabel('$\it \sigma_{yy} \; \rm (MPa)$','Interpreter','latex','FontSize',11)
box on

subplot(1,3,3)
x_min = min(sigma_xy_E/1e6);
x_max = max(sigma_xy_E/1e6);
hold on
line([x_min x_max ],[0 0],'LineStyle','-','Color',black,'LineWidth',0.5)
line([0 0],[y_min y_max ],'LineStyle','-','Color',black,'LineWidth',0.5)
plot(sigma_xy_E/1e6,y_EW,'-','Color',blue,'LineWidth',1.0)
plot(sigma_xy_W/1e6,y_EW,'--','Color',red,'LineWidth',1.0)
hold off
axis([x_min x_max y_min y_max])
xlabel('$\it \sigma_{xy} \; \rm (MPa)$','Interpreter','latex','FontSize',11)
box on

% Stresses at N and S boundaries:
figure
x_min = min(x_NS);
x_max = max(x_NS);
subplot(3,1,1)
y_min = min(sigma_xx_N/1e6);
y_max = max(sigma_xx_N/1e6);
hold on
plot(x_NS,sigma_xx_N/1e6,'-','Color',blue,'LineWidth',1.0)
plot(x_NS,sigma_xx_S/1e6,'--','Color',red,'LineWidth',1.0)
line([x_min x_max ],[0 0],'LineStyle','-','Color',black,'LineWidth',0.5)
line([0 0],[y_min y_max ],'LineStyle','-','Color',black,'LineWidth',0.5)
hold off
axis([x_min x_max y_min y_max])
if flag.plot_cap == 1
    x_text = x_min + 0.45 * (x_max-x_min);  
    y_text = y_max + 0.15 * (y_max-y_min);
    text(x_text,y_text,'plot\_fido','FontSize',10,'Color',red)
end
ylabel('$\it \sigma_{xx} \; \rm (MPa)$','Interpreter','latex','FontSize',11)
box on
legend('$N$','$S$','Interpreter','latex','FontSize',11,'location','SouthEast')

subplot(3,1,2)
y_min = min(sigma_yy_N/1e6);
y_max = max(sigma_yy_N/1e6);
hold on
line([x_min x_max ],[0 0],'LineStyle','-','Color',black,'LineWidth',0.5)
line([0 0],[y_min y_max ],'LineStyle','-','Color',black,'LineWidth',0.5)
plot(x_NS,sigma_yy_N/1e6,'-','Color',blue,'LineWidth',1.0)
plot(x_NS,sigma_yy_S/1e6,'--','Color',red,'LineWidth',1.0)
hold off
axis([x_min x_max y_min y_max])
ylabel('$\it \sigma_{yy} \; \rm (MPa)$','Interpreter','latex','FontSize',11)
box on

subplot(3,1,3)
y_min = min(sigma_xy_N/1e6);
y_max = max(sigma_xy_N/1e6);
hold on
line([x_min x_max ],[0 0],'LineStyle','-','Color',black,'LineWidth',0.5)
line([0 0],[y_min y_max ],'LineStyle','-','Color',black,'LineWidth',0.5)
plot(x_NS,sigma_xy_N/1e6,'-','Color',blue,'LineWidth',1.0)
plot(x_NS,sigma_xy_S/1e6,'--','Color',red,'LineWidth',1.0)
hold off
axis([x_min x_max y_min y_max])
xlabel('$\it x \; \rm (m)$','Interpreter','latex','FontSize',11)
ylabel('$\it \sigma_{xy} \; \rm (MPa)$','Interpreter','latex','FontSize',11)
box on

if flag.vert == 1 % vertical fault
    % Displacements at E and W boundaries:
    figure
    y_min = min(y_EW);
    y_max = max(y_EW);
    subplot(1,2,1)
    x_min = min( min(u_x_E*1e3), min(u_x_W*1e3) );
    x_max = max( max(u_x_E*1e3), max(u_x_W*1e3) );
    hold on
    line([x_min x_max ],[0 0],'LineStyle','-','Color',black,'LineWidth',0.5)
    line([0 0],[y_min y_max ],'LineStyle','-','Color',black,'LineWidth',0.5)
    plot(u_x_E*1e3,y_EW,'-','Color',blue,'LineWidth',1.0)
    plot(u_x_W*1e3,y_EW,'--','Color',red,'LineWidth',1.0)
    hold off
    if flag.plot_cap == 1
        x_text = x_min + 0.30 * (x_max-x_min);
        y_text = y_max + 0.05 * (y_max-y_min);
        text(x_text,y_text,'plot\_fido','FontSize',10,'Color',red)
    end
    axis([x_min x_max y_min y_max])
    xlabel('$\it u_x \; \rm (mm)$','Interpreter','latex','FontSize',11)
    ylabel('$\it y \; \rm (m)$','Interpreter','latex','FontSize',11)
    box on
    
    subplot(1,2,2)
    x_min = min( min(u_y_E*1e3), min(u_y_W*1e3) );
    x_max = max( max(u_y_E*1e3), max(u_y_W*1e3) );
    hold on
    plot(u_y_E*1e3,y_EW,'-','Color',blue,'LineWidth',1.0)
    plot(u_y_W*1e3,y_EW,'--','Color',red,'LineWidth',1.0)
    plot(u_y_E_check*1e3,y_EW,'o','MarkerIndices',1:25:length(y_EW),'MarkerEdgeColor',green)
    plot(u_y_W_check*1e3,y_EW,'*','MarkerIndices',1:25:length(y_EW),'MarkerEdgeColor',black)
    line([x_min x_max ],[0 0],'LineStyle','-','Color',black,'LineWidth',0.5)
    line([0 0],[y_min y_max ],'LineStyle','-','Color',black,'LineWidth',0.5)
    hold off
    axis([x_min x_max y_min y_max])
    xlabel('$\it u_y \; \rm (mm)$','Interpreter','latex','FontSize',11)
    box on
    legend('$E$','$W$','$E_{check}$','$W_{check}$','Interpreter','latex','FontSize',11,...
        'location','SouthEast')
    
    % Displacements at N and S boundaries:
    figure
    x_min = min(x_NS);
    x_max = max(x_NS);
    subplot(3,1,1)
    y_min = min( min(u_x_N*1e3), min(u_x_S*1e3) );
    y_max = max( max(u_x_N*1e3), max(u_x_S*1e3) );
    hold on
    plot(x_NS,u_x_N*1e3,'-','Color',blue,'LineWidth',1.0)
    plot(x_NS,u_x_S*1e3,'--','Color',red,'LineWidth',1.0)
    plot(x_NS,u_x_N_check*1e3,'o','MarkerIndices',1:25:length(y_EW),'MarkerEdgeColor',green)
    plot(x_NS,u_x_S_check*1e3,'*','MarkerIndices',1:25:length(y_EW),'MarkerEdgeColor',black)
    line([x_min x_max ],[0 0],'LineStyle','-','Color',black,'LineWidth',0.5)
    line([0 0],[y_min y_max ],'LineStyle','-','Color',black,'LineWidth',0.5)
    hold off
    axis([x_min x_max y_min y_max])
    if flag.plot_cap == 1
        x_text = x_min + 0.45 * (x_max-x_min);
        y_text = y_max + 0.15 * (y_max-y_min);
        text(x_text,y_text,'plot\_fido','FontSize',10,'Color',red)
    end
    ylabel('$\it u_x \; \rm (mm)$','Interpreter','latex','FontSize',11)
    box on
    legend('$N$','$S$','$N_{check}$','$S_{check}$','Interpreter','latex','FontSize',11,...
        'location','SouthEast')
    
    subplot(3,1,2)
    y_min = min(u_y_N*1e3);
    y_max = max(u_y_N*1e3);
    hold on
    line([x_min x_max ],[0 0],'LineStyle','-','Color',black,'LineWidth',0.5)
    line([0 0],[y_min y_max ],'LineStyle','-','Color',black,'LineWidth',0.5)
    plot(x_NS,u_y_N*1e3,'-','Color',blue,'LineWidth',1.0)
    hold off
    axis([x_min x_max y_min y_max])
    ylabel('$\it u_y \; \rm (mm)$','Interpreter','latex','FontSize',11)
    box on
    
    subplot(3,1,3)
    y_min = min(u_y_S*1e3);
    y_max = max(u_y_S*1e3);
    hold on
    line([x_min x_max ],[0 0],'LineStyle','-','Color',black,'LineWidth',0.5)
    line([0 0],[y_min y_max ],'LineStyle','-','Color',black,'LineWidth',0.5)
    plot(x_NS,u_y_S*1e3,'--','Color',red,'LineWidth',1.0)
    hold off
    axis([x_min x_max y_min y_max])
    ylabel('$\it u_y \; \rm (mm)$','Interpreter','latex','FontSize',11)
    box on
end
