function [sigma,p_0,p_comb,p_comb_full,y] = ifs_sigma(flag,model,numpar,delta,n_pt_y,p_inc,...
    y_delta,y_max,y_min)
%
% Computes various stresses at an inclined displaced fault for vertical coordinate y in an
% infinite-width reservoir.  
%

if flag.inf == 0 % finite reservoir
    [sigma,p_0,p_comb,p_comb_full,y] = ifs_sigma_finite(flag,model,numpar,delta,n_pt_y,....
        p_inc,y_delta,y_max,y_min);
else % infinite reservoir
    if flag.Buijze == 1 % Example from Buijze et al. (2017)
        [sigma,p_0,p_comb,p_comb_full,y] = ifs_sigma_BuijzeEtAl(flag,model,numpar,delta,...
            n_pt_y,p_inc,y_delta,y_max,y_min);
    else
        % Retrieve:
        a = model.a;
        b = model.b;
        alpha = model.alpha;
        delta_c = model.delta_c;
        D_center = model.D_center;
        g = model.g;
        kappa = model.kappa;
        K_eff = model.K_eff;
        mu_dyn = model.mu_dyn;
        mu_st = model.mu_st;
        nu = model.nu;
        p_0_center = model.p_0_center;
        phi = model.phi;
        rho_f = model.rho_f;
        rho_s = model.rho_s;
        sigma_v_0 = model.sigma_v_0;
        theta = model.theta;
        eps_reg = numpar.eps_reg;
        eta_reg = numpar.eta_reg;

        % Compute auxiliary variables:
        C = ((1-2*nu)*alpha*p_inc) / (2*pi*(1-nu)); % scaling parameter, N/m^2
        if y_max == y_min % input given as scalar or vector
            y = y_max;
        else % input given as minimum and maximum values
            y = linspace(y_min,y_max,n_pt_y);
        end

        % Compute gradient:
        if flag.depdep == 1 % depth-dependent initial pressure
            p_0 = p_0_center - rho_f * g * y; % p_0_center is the initial pressure @ y=0
        else % depth-independent initial pressure
            p_0 = p_0_center;
        end

        % Compute initial stresses:
        if sigma_v_0 == 0 % vertical stress is computed from the weigth of the overburden
            if flag.depdep == 1 % depth-dependent initial vertical stress
                sigma_v_0 = ( (1-phi)*rho_s + phi*rho_f ) * g * (y-D_center);
            else % depth-independent initial vertical stress
                sigma_v_0 = ( (1-phi)*rho_s + phi*rho_f ) * g * (-D_center);
            end
        end % Note: if sigma_v_0 ~= 0, the initial vertical stress for all y values is ...
            % equal to sigma_v_0.
        sigma_v_eff_0 = sigma_v_0 + alpha * p_0; % initial effective vertical stress
        sigma_h_eff_0 = K_eff * sigma_v_eff_0; % initial effective horizontal stress
        sigma_h_0 = sigma_h_eff_0 - alpha * p_0; % initial horizontal stress
        sigma_norm_0  = sigma_h_0 * (sin(theta))^2 + sigma_v_0 * (cos(theta))^2; % initial ...
            % normal stress
        sigma_shear_0 = - (sigma_v_0 - sigma_h_0) * sin(theta) * cos(theta); % initial ...
            % shear stress
        % Compute incremental stresses:
        if flag.reg == 1 % regularized expressions
            e = eta_reg;
            sigma_xx = -C * ( atan2( ((a+b)*e),(e^2+(y-b).*(y+a)) ) + ...
                              atan2( ((a+b)*e),(e^2+(y-a).*(y+b)) ) );
            sigma_xy = (C/2) * log( ( (e^2+(y-a).^2).*(e^2+(y+a).^2) ) ./ ...
                                    ( (e^2+(y-b).^2).*(e^2+(y+b).^2) ) );
        else % non-regularized expressions (containing discontinuities and singularities)
            sigma_xx = ...
                 0      * ( (y <= -b) + (b <= y) ) + ...
                -pi*C   * ( ( (-b < y) .* (y <= -a) ) + ( (a <= y) .* (y < b) ) ) + ...
                -2*pi*C * (-a < y) .* (y < a);
            sigma_xy = (C/2) * log( ((y-a).^2.*(y+a).^2)./((y-b).^2.*(y+b).^2) );
        end
        sigma_norm =  (-sigma_xy * sin(theta)*cos(theta) + sigma_xx * sin(theta)^2 );
        sigma_shear = ( sigma_xy * sin(theta)^2 + sigma_xx * sin(theta)*cos(theta) );

        % Compute effective stresses:
        sigma_norm_eff_0 = sigma_norm_0 + alpha*p_0; % effective initial normal stress
        if flag.reg == 1 % regularized expressions
            %    p_shift = alpha*p_inc/pi * atan2((a+b)*e,e^2+(y-b).*(y+b));
            p_shift = alpha*p_inc/pi * atan2((2*b)*e,e^2+(y-b).*(y+b)); % corrected April 2023
        else
            p_shift = alpha*p_inc * (-b < y) .* (y < b);
        end
        sigma_norm_eff = sigma_norm + p_shift; % regularized effective incremental normal ...
            % stress; only fault in reservoir "sees" incremental pressure
        sigma_norm_eff_full = sigma_norm + alpha*p_inc; % effective incremental normal ...
            % stress; full fault "sees" incremental pressure

        % Compute combined quantities:
        p_comb = p_0 + p_inc * (-b < y) .* (y < b); % combined initial and reservoir-only ...
            % incremental pressure
        p_comb_full = p_0 + p_inc; % combined initial and full-fault incremental pressures
        sigma_norm_comb = sigma_norm_0 + sigma_norm; % combined initial and incremental ...
            % normal stresses
        sigma_norm_eff_comb = sigma_norm_eff_0 + sigma_norm_eff; % combined initial and ...
            % incremental effective normal stresses; reservoir-only incremental pressure
        sigma_norm_eff_full_comb = sigma_norm_eff_0 + sigma_norm_eff_full; % combined ...
            % initial and incremental effective normal stresses; full-fault incremental pr.
        sigma_shear_comb = sigma_shear_0 + sigma_shear;  % combined initial and incremental ...
            % shear stresses

        % Compute right-hand slip boundary and Coulomb stresses:
        save = 0;
        if delta == 0
            save = flag.fric;
            flag.fric = 1; % local value
        end
        switch flag.fric
            case 0 % no friction
                sigma_slip = 0; % slip boundary; reservoir-only incremental pressure
                sigma_slip_full = 0; % slip boundary; full-fault incremental pressure
            case 1 % static friction only
                sigma_slip = kappa-mu_st*sigma_norm_eff_comb; % slip boundary; reservoir-...
                % only incremental pressure
% % %                 %%% Statement to test effect  of increased asymmetry.
% % %                 %%% Run for -24 < p < -30 MPa; regular coupled simulation 
% % %                 sigma_slip = sigma_slip .* (1 + 0.0005*y+0.005*sin(y)); %%% 
% % %                 %%%
                sigma_slip_full = kappa-mu_st*sigma_norm_eff_full_comb; % slip boundary; ...
                % full-fault incremental pressure
            case 2 % slip-dependent friction
                if flag.int ~= 3
% % %                     delta_fric = interp1(y_delta,delta,y,'linear',0);
                    delta_fric = interp1(y_delta,delta,y,'linear','extrap');
                else % forward method: discrete Chebyshev polynomials
                    delta_fric = delta;
                end
                df = abs(delta_fric);
                if flag.reg == 1 && eps_reg ~= 0 % regularized expressions
                    a1 = mu_st;
                    b1 = 0;
                    c1 = (mu_st-mu_dyn)/(2*eps_reg^2-2*delta_c*eps_reg);
                    a2 = (eps_reg*(mu_st+mu_dyn)-2*delta_c*mu_st)/(2*eps_reg-2*delta_c);
                    b2 = (mu_st-mu_dyn)/(eps_reg-delta_c);
                    a3 = (delta_c^2*(mu_dyn-mu_st)+...
                          2*eps_reg^2*mu_dyn-2*delta_c*eps_reg*mu_dyn)/...
                         (2*eps_reg^2-2*delta_c*eps_reg);
                    b3 = (delta_c*(mu_st-mu_dyn))/(eps_reg^2-delta_c*eps_reg);
                    c3 = -(mu_st-mu_dyn)/(2*eps_reg^2-2*delta_c*eps_reg);
                    mu = (a1+b1*df+c1*df.^2) .* (df < eps_reg) + ...
                         (a2+b2*df)          .* (eps_reg <= df) .* (df < delta_c-eps_reg) + ...
                         (a3+b3*df+c3*df.^2) .* (delta_c-eps_reg <= df) .* (df < delta_c) + ...
                        mu_dyn              .* (delta_c <= df);
                else
                    mu_help = mu_st - (mu_st - mu_dyn) * df/delta_c;
                    mu = min(max(mu_help,mu_dyn),mu_st); % ensures that mu_dyn < mu < mu_st
                end
                sigma_slip = kappa-mu.*sigma_norm_eff_comb; % slip boundary; reservoir-only ...
                    % incremental pressure
                sigma_slip_full = kappa-mu.*sigma_norm_eff_full_comb; % sl. b.; full-fault ...
                    % incremental pressure
        end
        if save ~=0
            flag.fric = save; % resets to original value
        end
        sigma_C = sigma_shear_comb - sigma_slip; % Coulomb stress; reservoir-only ...
        % incremental pressure
        sigma_C_full = sigma_shear_comb - sigma_slip_full; % Coulomb stress; full-fault ...
        % incremental pressure

        % Collect output:
        sigma.C = sigma_C;
        sigma.C_full = sigma_C_full;
        sigma.norm_0 = sigma_norm_0;
        sigma.norm = sigma_norm;
        sigma.norm_comb = sigma_norm_comb;
        sigma.norm_eff_0 = sigma_norm_eff_0;
        sigma.norm_eff = sigma_norm_eff;
        sigma.norm_eff_comb = sigma_norm_eff_comb;
        sigma.norm_eff_full = sigma_norm_eff_full;
        sigma.norm_eff_full_comb = sigma_norm_eff_full_comb;
        sigma.shear_0 = sigma_shear_0;
        sigma.shear = sigma_shear;
        sigma.shear_comb = sigma_shear_comb;
        sigma.p_shift = p_shift;
        sigma.slip = sigma_slip;
        sigma.slip_full = sigma_slip_full;
    end
end

