function [sigma,p_0,p_comb,p_comb_full,y] = ifs_sigma_finite(flag,model,numpar,delta,...
    n_pt_y,p_inc,y_delta,y_max,y_min)
%
% Computes various stresses at an inclined displaced fault for vertical coordinate y in a
% finite-width reservoir. 
%

% Retrieve:
a = model.a;
b = model.b;
c = model.c;
d = model.d;
alpha = model.alpha;
delta_c = model.delta_c;
D_center = model.D_center;
g = model.g;
kappa = model.kappa;
K_eff = model.K_eff;
mu_dyn = model.mu_dyn;
mu_st = model.mu_st;
nu = model.nu;
p_0_center = model.p_0_center;
phi = model.phi;
rho_f = model.rho_f;
rho_s = model.rho_s;
sigma_v_0 = model.sigma_v_0;
theta = model.theta;

eps_reg = numpar.eps_reg;
eta_reg = numpar.eta_reg;

% Compute auxiliary variables:
C = ((1-2*nu)*alpha*p_inc) / (2*pi*(1-nu)); % scaling parameter, N/m^2
if y_max == y_min % input given as scalar or vector
    y = y_max; 
else % input given as minimum and maximum values
    y = linspace(y_min,y_max,n_pt_y); 
end

% Compute gradient:
if flag.depdep == 1 % depth-dependent initial pressure
    p_0 = p_0_center - rho_f * g * y; % p_0_center is the initial pressure @ y=0
else % depth-independent initial pressure
    p_0 = p_0_center; 
end    
    
% Compute initial stresses:
if sigma_v_0 == 0 % vertical stress is computed from the weigth of the overburden
    if flag.depdep == 1 % depth-dependent initial vertical stress
        sigma_v_0 = ( (1-phi)*rho_s + phi*rho_f ) * g * (y-D_center);
    else % depth-independent initial vertical stress
        sigma_v_0 = ( (1-phi)*rho_s + phi*rho_f ) * g * (-D_center);
    end
end % Note: if sigma_v_0 ~= 0, the initial vertical stress for all y values is ...
    % equal to sigma_v_0.
sigma_v_eff_0 = sigma_v_0 + alpha * p_0; % initial effective vertical stress
sigma_h_eff_0 = K_eff * sigma_v_eff_0; % initial effective horizontal stress
sigma_h_0 = sigma_h_eff_0 - alpha * p_0; % initial horizontal stress
sigma_norm_0  = sigma_h_0 * (sin(theta))^2 + sigma_v_0 * (cos(theta))^2; % initial normal str.
sigma_shear_0 = - (sigma_v_0 - sigma_h_0) * sin(theta) * cos(theta); % initial shear stress

% Compute incremental stresses:
if flag.reg == 1 % regularized expressions
    small = 1e-9;
    x = y*cot(theta) + small; % x values just to the right of the fault
    G_xx_fin = Green_incl_g_xx(a,b,c,d,theta,x,y); % finite reservoir
    G_xx_inf = Green_incl_g_xx_inf(a,b,theta,x,y); % infinite reservoir
    Delta_G_xx = G_xx_fin - G_xx_inf; % difference: contribution from finite boundaries
    G_yy_fin = Green_incl_g_yy(a,b,c,d,theta,x,y);
    G_yy_inf = Green_incl_g_yy_inf(a,b,theta,x,y);
    Delta_G_yy = G_yy_fin - G_yy_inf;
    G_xy_fin = Green_incl_g_xy(a,b,c,d,theta,x,y);
    G_xy_inf = Green_incl_g_xy_inf(a,b,theta,x,y);
    Delta_G_xy = G_xy_fin - G_xy_inf;
    Delta_G_norm = Delta_G_xx * sin(theta)^2 + ...
        Delta_G_yy * cos(theta)^2 - ...
        Delta_G_xy * 2 * sin(theta)*cos(theta);
    Delta_G_shear = Delta_G_xy * ( sin(theta)^2-cos(theta)^2 ) + ...
        ( Delta_G_xx - Delta_G_yy ) * sin(theta)*cos(theta);
    Delta_sigma_norm = C * Delta_G_norm;
    Delta_sigma_shear = C * Delta_G_shear;
    e = eta_reg;
    sigma_xx = -C * ( atan2( ((a+b)*e),(e^2+(y-b).*(y+a)) ) + ...
        atan2( ((a+b)*e),(e^2+(y-a).*(y+b)) ) ); % just as for infinite case
    sigma_xy = (C/2) * log( ( (e^2+(y-a).^2).*(e^2+(y+a).^2) ) ./ ...
        ( (e^2+(y-b).^2).*(e^2+(y+b).^2) ) );
    sigma_norm =  (-sigma_xy * sin(theta)*cos(theta) + sigma_xx * sin(theta)^2 ) + ...
        Delta_sigma_norm; % just as for infinite case plus contrib. from finite boundaries
    sigma_shear = ( sigma_xy * sin(theta)^2 + sigma_xx * sin(theta)*cos(theta) ) + ...
        Delta_sigma_shear;
else % non-regularized expressions (containing discontinuities and singularities)
    small = 1e-12;
    x = y*cot(theta) + small; % x values just to the right of the fault
    G_xx = Green_incl_g_xx(a,b,c,d,theta,x,y);
    G_yy = Green_incl_g_yy(a,b,c,d,theta,x,y);
    G_xy = Green_incl_g_xy(a,b,c,d,theta,x,y);
    G_norm  = G_xx * (sin(theta))^2 + G_yy * (cos(theta))^2 - ...
        2*G_xy * sin(theta) * cos(theta);
    G_norm_minus2pi = G_norm - 2*pi * (-a < y) .* (y < b);
    G_shear = G_xy * ( (sin(theta))^2 - (cos(theta))^2 ) + ...
        (G_xx - G_yy) * sin(theta) * cos(theta);
    sigma_norm = C * G_norm_minus2pi;
    sigma_shear = C * G_shear;
end

% Compute effective stresses:
sigma_norm_eff_0 = sigma_norm_0 + alpha*p_0; % effective initial normal stress
sigma_norm_eff = sigma_norm + alpha*p_inc * (-b < y) .* (y < b); % effective ...
    % incremental normal stress; only fault in reservoir "sees" incremental pressure
sigma_norm_eff_full = sigma_norm + alpha*p_inc; % effective incremental normal stress; ...
    % full fault "sees" incremental pressure

% Compute combined quantities:
p_comb = p_0 + p_inc * (-b < y) .* (y < b); % combined initial and reservoir-only incr. pr.;
p_comb_full = p_0 + p_inc; % combined initial and full-fault incremental pressures
sigma_norm_comb = sigma_norm_0 + sigma_norm; % combined initial and incremental normal stresses 
sigma_norm_eff_comb = sigma_norm_eff_0 + sigma_norm_eff; % combined initial and incremental
% effective normal stresses; reservoir-only incremental pressure
sigma_norm_eff_full_comb = sigma_norm_eff_0 + sigma_norm_eff_full; % combined initial and
% incremental effective normal stresses; full-fault incremental pressure
sigma_shear_comb = sigma_shear_0 + sigma_shear;  % combined initial and incremental shear str.

% Compute right-hand slip boundary and Coulomb stresses:
switch flag.fric
    case 0 % no friction
        sigma_slip = 0; % slip boundary; reservoir-only incremental pressure
        sigma_slip_full = 0; % slip boundary; full-fault incremental pressure
    case 1 % static friction only
        sigma_slip = kappa-mu_st*sigma_norm_eff_comb; % slip boundary; reservoir-only incr. pr.
        sigma_slip_full = kappa-mu_st*sigma_norm_eff_full_comb; % sl. b.; full-fault incr. pr.
    case 2 % slip-dependent friction
        delta_fric = interp1(y_delta,delta,y,'linear',0);
        df = abs(delta_fric);
        if flag.reg == 1 && eps_reg ~= 0 % regularized expressions
            a1 = mu_st;
            b1 = 0;
            c1 = (mu_st-mu_dyn)/(2*eps_reg^2-2*delta_c*eps_reg);
            a2 = (eps_reg*(mu_st+mu_dyn)-2*delta_c*mu_st)/(2*eps_reg-2*delta_c);
            b2 = (mu_st-mu_dyn)/(eps_reg-delta_c);
            a3 = (delta_c^2*(mu_dyn-mu_st)+2*eps_reg^2*mu_dyn-2*delta_c*eps_reg*mu_dyn)/...
                (2*eps_reg^2-2*delta_c*eps_reg);
            b3 = (delta_c*(mu_st-mu_dyn))/(eps_reg^2-delta_c*eps_reg);
            c3 = -(mu_st-mu_dyn)/(2*eps_reg^2-2*delta_c*eps_reg);
            mu = (a1+b1*df+c1*df.^2) .* (df < eps_reg) + ...
                 (a2+b2*df)          .* (eps_reg <= df) .* (df < delta_c-eps_reg) + ...
                 (a3+b3*df+c3*df.^2) .* (delta_c-eps_reg <= df) .* (df < delta_c) + ...
                 mu_dyn              .* (delta_c <= df);
        else
            mu_help = mu_st - (mu_st - mu_dyn) * df/delta_c;
            mu = min(max(mu_help,mu_dyn),mu_st); % ensures that mu_dyn < mu < mu_st
         end
        sigma_slip = kappa-mu.*sigma_norm_eff_comb; % slip boundary; reservoir-only incr. pr.
        sigma_slip_full = kappa-mu.*sigma_norm_eff_full_comb; % sl. b.; full-fault incr. pr.
end
sigma_C = sigma_shear_comb - sigma_slip; % Coulomb stress; reservoir-only incremental pressure
sigma_C_full = sigma_shear_comb - sigma_slip_full; % Coulomb stress; full-fault incremental pr.

% Collect output:
sigma.C = sigma_C;
sigma.C_full = sigma_C_full;
sigma.norm_0 = sigma_norm_0;
sigma.norm = sigma_norm;
sigma.norm_comb = sigma_norm_comb;
sigma.norm_eff_0 = sigma_norm_eff_0;
sigma.norm_eff = sigma_norm_eff;
sigma.norm_eff_comb = sigma_norm_eff_comb;
sigma.norm_eff_full = sigma_norm_eff_full;
sigma.norm_eff_full_comb = sigma_norm_eff_full_comb;
sigma.shear_0 = sigma_shear_0;
sigma.shear = sigma_shear;
sigma.shear_comb = sigma_shear_comb;
sigma.slip = sigma_slip;
sigma.slip_full = sigma_slip_full;

