function [numpar,delta,grad,y_delta,y_til] = ifs_y_til(flag,model,numpar,plotpar,p_inc,y_til_0)
%
% Computes the slip patch boundaries y_til(i), i = 1,...,4, the slip gradient grad_delta and
% the slip delta. See ifs_auxiliary.m for the initial setting of flag.sim_type wich determines
% if a single-patch (flag.sim_type == 1), double patch (flag.sim_type == 2) or merged patch
% (flag.sim_type == 3) solution is computed.   
%

% Set solver options:
% options = optimoptions(@fsolve,'display','off'); % overwrites default options for fsolve ...
%     % to suppress output; change to 'iter', 'iter'-detailed', 'final' or 'final-detailed' for
%     % debugging

% options = optimoptions(@fsolve,'display','off','Algorithm','trust-region'); % overwrites ...
%     % default options for fsolve

options = optimoptions(@fsolve,'display','off','Algorithm','trust-region-dogleg'); % ...
    % overwrites default options for fsolve

% options = optimoptions(@fsolve,'display','off','Algorithm','levenberg-marquardt'); % ...
%     % overwrites default options for fsolve

% options = optimoptions(@fsolve,'display','off','StepTolerance',1e-12,...
%     'FunctionTolerance',1e-12); % overwrites default options for fsolve; tighter tolerance

if flag.echo == 2
    fprintf('Solving for slip patch boundaries\r');
end

switch flag.sim_type % (see ifs_auxiliary.m for setting of this flag)
    case {1,2} % uncoupled or coupled
        switch flag.fric
            case 1 % constant friction
                if flag.sym == 1 % indicates symmetry around y = 0
                    y_til_sym = fsolve(@(y_loc) ifs_resid_bounds_sym(flag,model,numpar,...
                        plotpar,0,p_inc,0,y_loc),y_til_0(3:4),options);
                    y_til = [-flip(y_til_sym)',y_til_sym']'; 
                    [delta,grad,y_delta] = ifs_delta_sym(flag,model,numpar,plotpar,0,p_inc,...
                        0,y_til);
                else % asymmetric configuration
                    y_til = fsolve(@(y_loc) ifs_resid_bounds(flag,model,numpar,plotpar,...
                        0,p_inc,0,y_loc),y_til_0,options);
                    [delta,grad,y_delta] = ifs_delta(flag,model,numpar,plotpar,0,p_inc,0,...
                        y_til);
% % %                     ifs_test_integrals(numpar,y_til); %%% test file %%%
% % %                     y_test = [-60 -10 20 50]; %%%
% % %                     ifs_test_integrals(numpar,y_test); %%% test file %%%                    
                end
            case 2 % slip-weakening friction
                switch flag.int
                    case{1,2} % inverse method: Gauss-Chebyshev quadrature, or Matlab's ...
                        % variable-support trapezium rule
                        i_max = numpar.n_iter_fric_max;  % max. allowed nr. of friction its., -
                        tol = numpar.tol_fric; % relative conv. tolerance for friction its., -
                        f_damp = numpar.f_damp_fric; % damping factor for iterative ...
                        % updating of slip patch boundaries; number between 0 and 1, -
                        i_iter = 0;
                        converged = 0; % flag to indicate convergence
                        delta = 0;
                        y_delta = 0;
                        y_til = y_til_0;
                        while converged == 0
                            i_iter = i_iter + 1;
                            if i_iter + 1 > i_max
                                if flag.cont == 1
                                    fprintf('Maximum allowed number of friction iterations exceeded; forced continuation\r');
                                else
                                    error('Maximum allowed number of friction iterations exceeded.')
                                end
                            end
                            [delta,grad,y_delta] = ifs_delta(flag,model,numpar,plotpar,...
                                delta,p_inc,y_delta,y_til);
                            y_til_old = y_til;
                            y_til_new = fsolve(@(y_loc) ifs_resid_bounds(flag,model,numpar,...
                                plotpar,delta,p_inc,y_delta,y_loc),y_til,options);
                            y_til = f_damp*y_til_old + (1-f_damp)*y_til_new;
                            if norm((y_til-y_til_old)./y_til_old) < tol || i_iter+1 > i_max
                                converged = 1;
                                if flag.echo == 2
                                    fprintf('Number of friction iterations: %-3.0i\r',i_iter);
                                end
                            end
                        end
                    case 3 % forward method: discrete Chebyshev polynomials
                    y_til = fsolve(@(y_loc) ifs_resid_bounds(flag,model,numpar,plotpar,...
                        0,p_inc,0,y_loc),y_til_0,options);
                    [delta,grad,y_delta] = ifs_delta(flag,model,numpar,plotpar,0,p_inc,0,...
                        y_til);                        
                end
        end
       
    case 3 % merged
        if flag.sym == 1 % indicates symmetry around y = 0
            y_til(4) = fsolve(@(y_loc) ifs_resid_bounds_sym(flag,model,numpar,plotpar,0,...
                p_inc,0,y_loc),y_til_0(4),options);
            y_til(1) = -y_til(4);
            [delta,grad,y_delta] = ifs_delta_sym(flag,model,numpar,plotpar,0,p_inc,0,...
                y_til(4));
        else % asymmetric configuration
            y_til(1:3:4) = fsolve(@(y_loc) ifs_resid_bounds(flag,model,numpar,plotpar,0,...
                p_inc,0,y_loc),y_til_0(1:3:4),options);
            [delta,grad,y_delta] = ifs_delta(flag,model,numpar,plotpar,0,p_inc,0,y_til(1:3:4));
        end
end
y_til = sort(y_til); % to correct for occasional swapping of values in fsolve 
